      PROGRAM BOSON                                                     ABTI0856
C    *******************************************************************ABTI0857
C    *                                                                 *ABTI0858
C    * PERFORM THE QUANTUM-MECHANICAL COMPLEMENT TO THE CLASSICAL STEP *ABTI0859
C    * OF THE DIELECTRIC THEORY OF EELS IN SPECULAR GEOMETRY USING A   *ABTI0860
C    * SUITABLE THERMODYNAMICAL AVERAGE OF THE QUANTIZED SURFACE       *ABTI0861
C    * HARMONIC OSCILLATORS                                            *ABTI0862
C    *                                                                 *ABTI0863
C    *******************************************************************ABTI0864
      PARAMETER(MMAX=14,NMAX=2**MMAX)                                   ABTI0865
      LOGICAL PICM,PICP                                                 ABTI0866
      CHARACTER TITLE1*48,TITLE2*72                                     ABTI0867
      DIMENSION P1(NMAX),P2(NMAX),R1(NMAX),R2(NMAX)                     ABTI0868
*                                                                       ABTI0869
*     REMARK : THE TWO ARRAYS R1 AND R2 ARE USED FOR FOURIER            ABTI0870
*     TRANSFORMING THE USER-SUPPLIED INSTRUMENTAL RESPONSE OF THE       ABTI0871
*     SPECTROMETER THAT HAS TO BE CODED IN THE EXTERNAL ROUTINE         ABTI0872
*     RESPON CALLED WHEN THE INPUT PARAMETER GAUSS IS < 0 OR > 1.       ABTI0873
*     WITH 0 <= GAUSS <= 1, R1 AND R2 ARE NOT USED.                     ABTI0874
*                                                                       ABTI0875
C *** RATIONAL APPROXIMATIONS FOR EI(U)*EXP(-U) + E1(U)*EXP(+U)         ABTI0876
C *** IN THE INTERVALS (0,1.3) AND (1.3,INFINITY) <ACCURACY : 4.E-04>   ABTI0877
C *** USED FOR FOURIER TRANSFORMING HALF-LORENTZIAN FUNCTIONS           ABTI0878
      O1(U) = -SINH(U)*LOG(U**2) +                                      ABTI0879
     1         U*((0.03114*U**2 + 0.41666)*U**2 + 0.84557)              ABTI0880
      O2(U) = (((202.91/U**2 + 932.21)/U**2 + 41.740)/U**2 + 2.0)/      ABTI0881
     2        (((540.88/U**2 + 345.67)/U**2 + 18.961)/U**2 + 1.0)/U     ABTI0882
C                                                                       ABTI0883
      DATA FM1/0.0/,FP1/0.0/                                            ABTI0884
      PI = 4.0*ATAN(1.0)                                                ABTI0885
C                                                                       ABTI0886
C *** READ INPUT PARAMETERS                                             ABTI0887
C                                                                       ABTI0888
      OPEN(UNIT=13,FILE ='BOSIN')                                       ABTI0889
C     TARGET TEMPERATURE (KELVIN)                                       ABTI0890
      READ(13,*) T                                                      ABTI0891
C     WIDTH OF THE INSTRUMENTAL RESPONSE (CM**-1)                       ABTI0892
      READ(13,*) WIDTH                                                  ABTI0893
C     FRACTION OF GAUSSIAN FOR THE INSTRUMENTAL RESPONSE                ABTI0894
      READ(13,*) GAUSS                                                  ABTI0895
C     ASYMMETRY OF THE INSTRUMENTAL RESPONSE                            ABTI0896
      READ(13,*) ASYM                                                   ABTI0897
C     LOWER AND UPPER ENERGY LOSSES FOR THIS COMPUTATION (CM**-1)       ABTI0898
      READ(13,*) EMIN                                                   ABTI0899
      READ(13,*) EMAX                                                   ABTI0900
      CLOSE(UNIT=13)                                                    ABTI0901
      WRITE(6,101) T,WIDTH,GAUSS,ASYM,EMIN,EMAX                         ABTI0902
C                                                                       ABTI0903
C *** READ THE TABLE OF VALUES OF THE CLASSICAL LOSS SPECTRUM           ABTI0904
C                                                                       ABTI0905
      OPEN(UNIT=12,FILE='EELSOU')                                       ABTI0906
      READ(12,'(A48)') TITLE1                                           ABTI0907
      READ(12,102) TITLE2                                               ABTI0908
      N = 0                                                             ABTI0909
    1 READ(12,*,END=2) WMAX,Y                                           ABTI0910
      IF(WMAX.LT.0.0) GOTO 1                                            ABTI0911
      N = N+1                                                           ABTI0912
      P2(N) = Y                                                         ABTI0913
      IF(N.EQ.1) WMIN = WMAX                                            ABTI0914
      IF(N.LT.NMAX) GOTO 1                                              ABTI0915
    2 CLOSE(UNIT=12)                                                    ABTI0916
      IF(N.LE.0) STOP '*** NO INPUT VALUES FOR PCL ***'                 ABTI0917
      WRITE(6,103) TITLE2,N,WMIN,WMAX                                   ABTI0918
      DWN = (WMAX-WMIN)/(N-1)                                           ABTI0919
C                                                                       ABTI0920
C *** REDEFINE THE FREQUENCY INTERVAL (0,WMAX) TO BE USED FOR THE       ABTI0921
C *** FOURIER TRANSFORMS                                                ABTI0922
C                                                                       ABTI0923
      JMIN = INT(0.5+WMIN/DWN)                                          ABTI0924
      IF((JMIN-0.5)*DWN.LT.WMIN) JMIN = JMIN+1                          ABTI0925
      FAC = ((JMIN-0.5)*DWN-WMIN)/DWN                                   ABTI0926
      JMAX = INT(0.5+WMAX/DWN)                                          ABTI0927
      TEST = MAX(1.5*ABS(EMIN),1.5*ABS(EMAX),6.0*WMAX)                  ABTI0928
      N = 2                                                             ABTI0929
      DO 3 M=2,MMAX                                                     ABTI0930
         N = 2*N                                                        ABTI0931
         WMAX = N*DWN                                                   ABTI0932
         IF(WMAX.GE.TEST) GOTO 4                                        ABTI0933
    3 CONTINUE                                                          ABTI0934
      M = MMAX                                                          ABTI0935
      N = NMAX                                                          ABTI0936
      WMAX = N*DWN                                                      ABTI0937
      WRITE(6,104) NMAX,MMAX                                            ABTI0938
    4 WRITE(6,105) WMAX,DWN,N,M                                         ABTI0939
      DO 5 J=1,N                                                        ABTI0940
         P1(J) = 0.0                                                    ABTI0941
    5 CONTINUE                                                          ABTI0942
C                                                                       ABTI0943
C *** INTERPOLATE PCL ON A SUITABLE MESH IN (0,WMAX)                    ABTI0944
C                                                                       ABTI0945
      I = 1                                                             ABTI0946
      DO 6 J=JMIN,MIN(JMAX,N)                                           ABTI0947
         P1(J) = P2(I)+FAC*(P2(I+1)-P2(I))                              ABTI0948
         I = I+1                                                        ABTI0949
    6 CONTINUE                                                          ABTI0950
      DO 7 J=1,N                                                        ABTI0951
         P2(J) = P1(J)                                                  ABTI0952
    7 CONTINUE                                                          ABTI0953
C                                                                       ABTI0954
C *** CHARACTERISTIC FUNCTION F(TAU)                                    ABTI0955
C                                                                       ABTI0956
      CALL SICOT(P1,M,DWN,1.39*T)                                       ABTI0957
      CALL SINTR(P2,M,DWN)                                              ABTI0958
      H = (PI+PI)/WMAX                                                  ABTI0959
      IF(GAUSS.LT.0 .OR. GAUSS.GT.1.0) GOTO 12                          ABTI0960
C                                                                       ABTI0961
C *** BROADEN THE SPECTRUM BY CONVOLUTING THE CHARACTERISTIC FUNCTION BYABTI0962
C *** A WEIGHTED SUM OF A LORENTZIAN AND A GAUSSIAN RESPONSE FUNCTIONS  ABTI0963
C                                                                       ABTI0964
      ALFA = MAX(1.5*DWN,WIDTH)                                         ABTI0965
      IF(ALFA.GT.WIDTH) WRITE(6,106) ALFA                               ABTI0966
      IF(ALFA.GT.0.5*WMAX/PI) STOP                                      ABTI0967
     .       '*** WIDTH IS TOO LARGE, NOTHING DONE ***'                 ABTI0968
      ALFA = 0.5*ALFA                                                   ABTI0969
      SIGMA = ALFA/1.66511                                              ABTI0970
      A1 = (1.0-ASYM)/2.0*(1.0-GAUSS)                                   ABTI0971
      A2 = (1.0+ASYM)/2.0*(1.0-GAUSS)                                   ABTI0972
      IF(A1.LT.0.0 .OR. A2.LT.0.0) STOP                                 ABTI0973
     .       '*** INVALID INPUT : ASYM SHOULD BE IN (-1,+1) ***'        ABTI0974
      G1 = (1.0-ASYM)*ALFA                                              ABTI0975
      G2 = (1.0+ASYM)*ALFA                                              ABTI0976
      P1(1) = 1.0                                                       ABTI0977
      P2(1) = 0.0                                                       ABTI0978
      DO 11 I=2,N                                                       ABTI0979
         X = (I-1)*H                                                    ABTI0980
         FR = 0.0                                                       ABTI0981
         FI = 0.0                                                       ABTI0982
         IF(A1.EQ.0.0) GOTO 8                                           ABTI0983
         X1 = G1*X                                                      ABTI0984
         IF(X1.LE.100.0) FR = A1*EXP(-X1)                               ABTI0985
         IF(A1.EQ.A2) GOTO 8                                            ABTI0986
         IF(X1.LE.1.3)   FI = A1*O1(X1)                                 ABTI0987
         IF(X1.GT.1.3)   FI = A1*O2(X1)                                 ABTI0988
    8    IF(A2.EQ.0.0) GOTO 9                                           ABTI0989
         X2 = G2*X                                                      ABTI0990
         IF(X2.LE.100.0) FR = FR+A2*EXP(-X2)                            ABTI0991
         IF(A2.EQ.A1) GOTO 9                                            ABTI0992
         IF(X2.LE.1.3)   FI = FI-A2*O1(X2)                              ABTI0993
         IF(X2.GT.1.3)   FI = FI-A2*O2(X2)                              ABTI0994
    9    IF(GAUSS.EQ.0.0) GOTO 10                                       ABTI0995
         X3 = SIGMA*X                                                   ABTI0996
         IF(X3.LE.10.0) FR=FR+GAUSS*EXP(-X3**2)                         ABTI0997
   10    FI = FI/PI                                                     ABTI0998
         FAC = EXP(-P1(I))                                              ABTI0999
         CP2 = FAC*COS(P2(I))                                           ABTI1000
         SP2 = FAC*SIN(P2(I))                                           ABTI1001
         P1(I) = FR*CP2+FI*SP2                                          ABTI1002
         P2(I) = FI*CP2-FR*SP2                                          ABTI1003
   11 CONTINUE                                                          ABTI1004
      IF(ABS(FI).GT.1.0E-03) WRITE(6,107) DWN                           ABTI1005
      GOTO 15                                                           ABTI1006
C                                                                       ABTI1007
C *** BROADEN THE SPECTRUM BY CONVOLUTING THE CHARACTERISTIC FUNCTION   ABTI1008
C *** WITH A USER-SUPPLIED RESPONSE FUNCTION (ARBITRARY NORMALIZATION)  ABTI1009
C                                                                       ABTI1010
   12 WRITE(6,*) '==> SWITCH TO A USER-SUPPLIED INSTRUMENTAL RESPONSE'  ABTI1011
      ALFA = AMAX1(4.0*DWN,WIDTH)                                       ABTI1012
      IF(ALFA.GT.WIDTH) WRITE(6,106) ALFA                               ABTI1013
      IF(ALFA.LT.10.0*DWN) WRITE(6,107) DWN                             ABTI1014
C     MAKE A TABLE OF THE RESPONSE FUNCTION                             ABTI1015
      R1(1) = RESPON(0.0,ALFA)                                          ABTI1016
      R2(1) = R1(1)                                                     ABTI1017
      DO 13 I=2,N                                                       ABTI1018
         X = (I-1)*DWN                                                  ABTI1019
         R1(I) = RESPON(X,ALFA)                                         ABTI1020
         R2(I) = RESPON(-X,ALFA)                                        ABTI1021
   13 CONTINUE                                                          ABTI1022
C     FOURIER TRANSFORM IT                                              ABTI1023
      CALL RCFFI(R1,R2,-M,DWN)                                          ABTI1024
C     NORMALIZATION OF THE RESPONSE FUNCTION, AND CONVOLUTION           ABTI1025
      ANORM = R1(1)                                                     ABTI1026
      DO 14 I=1,N                                                       ABTI1027
         FAC = EXP(-P1(I))/ANORM                                        ABTI1028
         CP2 = FAC*COS(P2(I))                                           ABTI1029
         SP2 = FAC*SIN(P2(I))                                           ABTI1030
         P1(I) = R1(I)*CP2+R2(I)*SP2                                    ABTI1031
         P2(I) = R2(I)*CP2-R1(I)*SP2                                    ABTI1032
   14 CONTINUE                                                          ABTI1033
      ALFA = ALFA/2.0                                                   ABTI1034
C                                                                       ABTI1035
C *** FULL EELS SPECTRUM                                                ABTI1036
C                                                                       ABTI1037
   15 CALL RCFFI(P1,P2,M,H)                                             ABTI1038
C                                                                       ABTI1039
C *** OUTPUT                                                            ABTI1040
C                                                                       ABTI1041
      OPEN(UNIT=14,FILE='BOSOU')                                        ABTI1042
      ISTEP = MAX(INT(ALFA/DWN/10.0),1)                                 ABTI1043
      WRITE(14,108) TITLE1,T,GAUSS                                      ABTI1044
      WRITE(14,102) TITLE2                                              ABTI1045
      NOUT = 0                                                          ABTI1046
      IF(EMIN.GT.0.0) GOTO 18                                           ABTI1047
      IMIN = N-INT(ABS(EMIN)/DWN)                                       ABTI1048
      IF((IMIN-N)*DWN.LT.EMIN) IMIN = IMIN+1                            ABTI1049
      DO 17 I=IMIN,N                                                    ABTI1050
         J = N-I                                                        ABTI1051
         IF(MOD(J,ISTEP).GT.0) GOTO 17                                  ABTI1052
         X = -J*DWN                                                     ABTI1053
         IF(X.GT.EMAX) GOTO 20                                          ABTI1054
         NOUT = NOUT+1                                                  ABTI1055
         WRITE(14,109) X,P2(J+1)                                        ABTI1056
   17 CONTINUE                                                          ABTI1057
   18 IF(EMAX.LT.DWN) GOTO 20                                           ABTI1058
      IMAX = INT(EMAX/DWN)+1                                            ABTI1059
      IF((IMAX-1)*DWN.GT.EMAX) IMAX = IMAX-1                            ABTI1060
      IF(IMAX.LT.2) GOTO 20                                             ABTI1061
      DO 19 I=2,IMAX                                                    ABTI1062
         IF(MOD(I-1,ISTEP).GT.0) GOTO 19                                ABTI1063
         X = (I-1)*DWN                                                  ABTI1064
         IF(X.LT.EMIN) GOTO 19                                          ABTI1065
         NOUT = NOUT+1                                                  ABTI1066
         WRITE(14,109) X,P1(I)                                          ABTI1067
   19 CONTINUE                                                          ABTI1068
   20 CLOSE(UNIT=14)                                                    ABTI1069
      WRITE(6,110) NOUT,ISTEP*DWN                                       ABTI1070
C                                                                       ABTI1071
C *** ANALYZE THE SPECTRUM                                              ABTI1072
C                                                                       ABTI1073
      WRITE(6,111)                                                      ABTI1074
      WN = 0.0                                                          ABTI1075
      FM = P1(1)                                                        ABTI1076
      FP = P2(1)                                                        ABTI1077
      IF(P2(2).LT.FP .AND. P1(2).LT.FP) WRITE(6,112) WN,FP              ABTI1078
      FAC = 5.0E-06*FP*DWN                                              ABTI1079
      IMAX = 2+INT(AMAX1(ABS(EMIN),ABS(EMAX))/DWN)                      ABTI1080
      DO 25 I=2,IMAX                                                    ABTI1081
         FM0 = FM1                                                      ABTI1082
         FP0 = FP1                                                      ABTI1083
         FM1 = FM                                                       ABTI1084
         FP1 = FP                                                       ABTI1085
         FM = P2(I)                                                     ABTI1086
         FP = P1(I)                                                     ABTI1087
         IF(I.EQ.2) GOTO 25                                             ABTI1088
         PICM = .FALSE.                                                 ABTI1089
         PICP = .FALSE.                                                 ABTI1090
         WN = (I-1)*DWN                                                 ABTI1091
         IF(FM1-FM0.LT.0.0) GOTO 22                                     ABTI1092
         IF(FM1-FM.LT.0.0) GOTO 22                                      ABTI1093
         A = (FM1-FM0)+(FM1-FM)                                         ABTI1094
         IF(A.LT.FAC) GOTO 22                                           ABTI1095
         B = 0.5*((FM1-FM0)+3.0*(FM1-FM))                               ABTI1096
         U = B/A                                                        ABTI1097
         WMPIC = -WN+U*DWN                                              ABTI1098
         FMPIC = FM+0.5*B*U                                             ABTI1099
         PICM = .TRUE.                                                  ABTI1100
   22    IF(FP1-FP0.LT.0.0) GOTO 23                                     ABTI1101
         IF(FP1-FP.LT.0.0) GOTO 23                                      ABTI1102
         A = (FP1-FP0)+(FP1-FP)                                         ABTI1103
         IF(A.LT.FAC) GOTO 23                                           ABTI1104
         B = 0.5*((FP1-FP0)+3.0*(FP1-FP))                               ABTI1105
         U = B/A                                                        ABTI1106
         WPPIC = WN-U*DWN                                               ABTI1107
         FPPIC = FP+0.5*B*U                                             ABTI1108
         PICP = .TRUE.                                                  ABTI1109
         IF(PICP.AND.PICM) WRITE(6,112) WPPIC,FPPIC,WMPIC,FMPIC         ABTI1110
         IF(PICP.AND.(.NOT.PICM)) WRITE(6,112) WPPIC,FPPIC              ABTI1111
   23    IF(PICM.AND.(.NOT.PICP)) WRITE(6,113) WMPIC,FMPIC              ABTI1112
   25 CONTINUE                                                          ABTI1113
      STOP                                                              ABTI1114
  101 FORMAT(' PROGRAM BOSON (MARCH 1990)'/' T =',F6.1,' K ,  WIDTH =', ABTI1115
     ,       F7.2,' CM**-1'/' GAUSS =',F5.2,' ,  ASYM =',F5.2/          ABTI1116
     .       ' ENERGY LOSSES FROM',G11.4,'  TO',G11.4,' CM**-1')        ABTI1117
  102 FORMAT(A72)                                                       ABTI1118
  103 FORMAT(1X,A72/' READ',I6,' VALUES OF PCL FROM',G15.7,' TO',G15.7) ABTI1119
  104 FORMAT(' +++ N HAS BEEN FIXED AT',I6,' = 2**',I2,' +++')          ABTI1120
  105 FORMAT(' CLASSICAL SPECTRUM REDEFINED FROM 0.0 TO',G14.7/         ABTI1121
     ,' STEP SIZE =',G10.3,I10,' (= 2**',I2,') POINTS')                 ABTI1122
  106 FORMAT(' +++ WIDTH HAS BEEN ENLARGED TO',G11.4,' +++')            ABTI1123
  107 FORMAT(' ... POOR REPRESENTATION OF THE RESPONSE FUNCTION ...'/5X,ABTI1124
     ,'THE STEP SIZE (',G10.3,') SHOULD BE REDUCED')                    ABTI1125
  108 FORMAT(A48,'T =',F6.1,' GAUSS =',F5.2)                            ABTI1126
  109 FORMAT(2E15.7)                                                    ABTI1127
  110 FORMAT(/1X,I6,' VALUES WRITTEN ON DISK, STEP SIZE =',G10.3)       ABTI1128
  111 FORMAT(/2(' PEAK LOCATION    AMPLITUDE',5X))                      ABTI1129
  112 FORMAT(2(1X,F13.2,E13.4,5X))                                      ABTI1130
  113 FORMAT(33X,F13.2,E13.4)                                           ABTI1131
      END                                                               ABTI1132
      REAL FUNCTION RESPON(W,WIDTH)                                     ABTI1133
C   ******************************************************************* ABTI1134
C   *                                                                 * ABTI1135
C   * INSTRUMENTAL RESPONSE OF THE SPECTROMETER FOR THE FREQUENCY W   * ABTI1136
C   * WIDTH IS THE FULL WIDTH AT HALF MAXIMUM OF THE RESPONSE FUNCTION* ABTI1137
C   *                                                                 * ABTI1138
C   ******************************************************************* ABTI1139
      A = 0.8*WIDTH                                                     ABTI1140
      RESPON =0.0                                                       ABTI1141
      U = W/A                                                           ABTI1142
      IF(U.LE.-1.0 .OR. U.GE.2.0) RETURN                                ABTI1143
      IF(U.GT.0.0) GOTO 1                                               ABTI1144
      RESPON = 2.0*SQRT((U+1.0)**3)                                     ABTI1145
      RETURN                                                            ABTI1146
    1 IF(U.GT.1.0) GOTO 2                                               ABTI1147
      RESPON = 2.0-4.0*SQRT(U**3)+3.0*U                                 ABTI1148
      RETURN                                                            ABTI1149
    2 RESPON = 4.0+2.0*SQRT((U-1.0)**3)-3.0*U                           ABTI1150
      RETURN                                                            ABTI1151
      END                                                               ABTI1152
      SUBROUTINE SICOT(F,M,H,X0)                                        ABTI1153
C   ******************************************************************* ABTI1154
C   *                                                                 * ABTI1155
C   * INTEGRAL TRANSFORM G(Y,X0) OF A REAL FUNCTION F(X) DEFINED BY   * ABTI1156
C   *                                                                 * ABTI1157
C   * G(X0,Y) = INTEGRAL OF F(X)/TANH(X/X0)*(1-COS(X*Y)) DX           * ABTI1158
C   *                                                                 * ABTI1159
C   * F(X) IS TABULATED ON THE MESH OF POINTS XJ = (J-1/2)*H ,        * ABTI1160
C   * J = 1, 2, ..., N ,  WITH N = 2**M                               * ABTI1161
C   * G(X0,Y) IS COMPUTED ON THE MESH OF POINTS YK = K*2PI/(N*H) ,    * ABTI1162
C   * K = 0, 1, 2, ..., N-1                                           * ABTI1163
C   *                                                                 * ABTI1164
C   * INPUT ...                                                       * ABTI1165
C   * F(J) CONTAINS THE VALUE OF F(XJ) , J = 1, 2, ..., N             * ABTI1166
C   * M IS SUCH THAT N = 2**IABS(M)                                   * ABTI1167
C   * H IS THE STEP SIZE (MUST BE POSITIVE)                           * ABTI1168
C   *                                                                 * ABTI1169
C   * OUTPUT ...                                                      * ABTI1170
C   * F(K+1) CONTAINS G(X0,YK), K = 0, 1, ..., N-1                    * ABTI1171
C   *                                                                 * ABTI1172
C   * COMPUTING REMARKS ...                                           * ABTI1173
C   * F IS A REAL ARRAY WITH DIMENSION N OR MORE                      * ABTI1174
C   * IT IS SUPPOSED THAT F(X) IS ZERO FOR X > N*H                    * ABTI1175
C   * EXTERNAL REFERENCE : SINTR (SINE TRANSFORM OF A REAL FUNCTION). * ABTI1176
C   *                                                                 * ABTI1177
C   ******************************************************************* ABTI1178
      DIMENSION F(*)                                                    ABTI1179
      IF(M.EQ.0) THEN                                                   ABTI1180
         F(1) = 0.0D0                                                   ABTI1181
         RETURN                                                         ABTI1182
      ENDIF                                                             ABTI1183
      IF(H.LE.0.0 .OR. X0.LT.0.0) GOTO 9                                ABTI1184
      N = 2**IABS(M)                                                    ABTI1185
C                                                                       ABTI1186
C *** EVALUATE THE INTEGRAL FROM XJ TO INFINITY OF F(X)/TANH(X/X0) DX   ABTI1187
C     AND STORE THE RESULT IN F(J)                                      ABTI1188
C                                                                       ABTI1189
      C = 0.0                                                           ABTI1190
      IF(X0.GT.H/16.0) C = EXP(-H/X0)                                   ABTI1191
      S = 1.0-C                                                         ABTI1192
      E = C**2                                                          ABTI1193
      A = 0.25*H                                                        ABTI1194
      B = 0.25*X0                                                       ABTI1195
      DO 2 I=1,N                                                        ABTI1196
         IF(I.LT.N) F(I) = F(I)+F(I+1)                                  ABTI1197
         D = A                                                          ABTI1198
         IF(S.EQ.1.0) GOTO 1                                            ABTI1199
         SA = S                                                         ABTI1200
         C = C*E                                                        ABTI1201
         S = 1.0-C                                                      ABTI1202
         D = D+B*LOG(S/SA)                                              ABTI1203
    1    F(I) = F(I)*D                                                  ABTI1204
    2 CONTINUE                                                          ABTI1205
      J = N                                                             ABTI1206
      DO 3 I=2,N                                                        ABTI1207
         J = J-1                                                        ABTI1208
         F(J) = F(J)+F(J+1)                                             ABTI1209
    3 CONTINUE                                                          ABTI1210
      MSIGN = -IABS(M)                                                  ABTI1211
      CALL SINTR(F,MSIGN,H)                                             ABTI1212
      RETURN                                                            ABTI1213
    9 IF(H.LE.0.0) WRITE(6,10) H                                        ABTI1214
      IF(X0.LT.0.0) WRITE(6,11) X0                                      ABTI1215
   10 FORMAT(' *** INCORRECT STEP SIZE IN <SICOT>, H =',E15.6,' ***')   ABTI1216
   11 FORMAT(' *** INCORRECT INPUT IN <SICOT>, X0 =',E15.6,' ***')      ABTI1217
      STOP                                                              ABTI1218
      END                                                               ABTI1219
      SUBROUTINE SINTR(F,MSIGN,H)                                       ABTI1220
C   ******************************************************************* ABTI1221
C   *                                                                 * ABTI1222
C   * INTEGRAL SINE TRANSFORM OF A REAL FUNCTION F(X)                 * ABTI1223
C   *                                                                 * ABTI1224
C   * G(Y) = INTEGRAL FROM ZERO TO INFINITY OF F(X)*SIN(X*Y) DX       * ABTI1225
C   * IF MSIGN >= 0 OR                                                * ABTI1226
C   * G(Y) = Y * INTEGRAL FROM ZERO TO INFINITY OF F(X)*SIN(X*Y) DX   * ABTI1227
C   * IF MSIGN < 0                                                    * ABTI1228
C   *                                                                 * ABTI1229
C   * F(X) IS TABULATED ON THE MESH OF POINTS XJ = (J-1/2)*H ,        * ABTI1230
C   * J = 1, 2, ..., N ,  WITH N = 2**IABS(MSIG)                      * ABTI1231
C   * G(Y) IS COMPUTED ON THE MESH OF POINTS YK = K*2PI/(N*H) ,       * ABTI1232
C   * K = 0, 1, 2, ..., N-1                                           * ABTI1233
C   *                                                                 * ABTI1234
C   * INPUT ...                                                       * ABTI1235
C   * F(J) CONTAINS THE VALUE OF F(XJ) , J = 1, 2, ..., N             * ABTI1236
C   * MSIGN IS SUCH THAT N = 2**IABS(MSIGN)                           * ABTI1237
C   * H IS THE STEP SIZE (MUST BE POSITIVE)                           * ABTI1238
C   *                                                                 * ABTI1239
C   * OUTPUT ...                                                      * ABTI1240
C   * F(K+1) CONTAINS G(YK), K = 0, 1, ..., N-1                       * ABTI1241
C   *                                                                 * ABTI1242
C   * COMPUTING REMARKS ...                                           * ABTI1243
C   * F IS A REAL ARRAY WITH DIMENSION N OR MORE                      * ABTI1244
C   * IT IS SUPPOSED THAT F(X) IS ZERO FOR X > N*H                    * ABTI1245
C   *                                                                 * ABTI1246
C   ******************************************************************* ABTI1247
      DIMENSION F(*)                                                    ABTI1248
      IF(MSIGN.EQ.0) GOTO 16                                            ABTI1249
      IF(H.LE.0.0) GOTO 17                                              ABTI1250
C                                                                       ABTI1251
C *** FOR J AND K = 0, 1, 2 ... N/2-1, COMPUTE                          ABTI1252
C     S1 = SUM ( F(2*J+1)*COS(4*PI*K*J/N) - F(2*J+2)*SIN(4*PI*K*J/N) )  ABTI1253
C     S2 = SUM ( F(2*J+1)*SIN(4*PI*K*J/N) + F(2*J+2)*COS(4*PI*K*J/N) )  ABTI1254
C     STORE S1 IN F(2*K+1) AND S2 IN F(2*K+2)                           ABTI1255
C     (ADAPTED FROM CFFT ROUTINE, CERN LIBRARY MEMBER D704)             ABTI1256
C                                                                       ABTI1257
      M2 = IABS(MSIGN)-1                                                ABTI1258
      N = 2**M2                                                         ABTI1259
      IF(N.EQ.1) GOTO 10                                                ABTI1260
      NV2 = N/2                                                         ABTI1261
      NM1 = N-1                                                         ABTI1262
      J = 1                                                             ABTI1263
      DO 3 I=1,NM1                                                      ABTI1264
         IF(I.LT.J) THEN                                                ABTI1265
            I2 = I+I                                                    ABTI1266
            J2 = J+J                                                    ABTI1267
            TI = F(J2)                                                  ABTI1268
            F(J2) = F(I2)                                               ABTI1269
            F(I2) = TI                                                  ABTI1270
            I2 = I2-1                                                   ABTI1271
            J2 = J2-1                                                   ABTI1272
            TR = F(J2)                                                  ABTI1273
            F(J2) = F(I2)                                               ABTI1274
            F(I2) = TR                                                  ABTI1275
         ENDIF                                                          ABTI1276
         K = NV2                                                        ABTI1277
    1    IF(K.GE.J) GOTO 2                                              ABTI1278
         J = J-K                                                        ABTI1279
         K = K/2                                                        ABTI1280
         GOTO 1                                                         ABTI1281
    2    J = J+K                                                        ABTI1282
    3 CONTINUE                                                          ABTI1283
      DO 4 I=1,N,2                                                      ABTI1284
         I2 = I+I                                                       ABTI1285
         TI = F(I2+2)                                                   ABTI1286
         F(I2+2) = F(I2)-TI                                             ABTI1287
         F(I2) = F(I2)+TI                                               ABTI1288
         I2 = I2-1                                                      ABTI1289
         TR = F(I2+2)                                                   ABTI1290
         F(I2+2) = F(I2)-TR                                             ABTI1291
         F(I2) = F(I2)+TR                                               ABTI1292
    4 CONTINUE                                                          ABTI1293
      IF(M2.EQ.1) GOTO 10                                               ABTI1294
      C = 0.0                                                           ABTI1295
      S = 1.0                                                           ABTI1296
      LE = 2                                                            ABTI1297
      DO 8 L=2,M2                                                       ABTI1298
         WR = C                                                         ABTI1299
         WI = S                                                         ABTI1300
         UR = WR                                                        ABTI1301
         UI = WI                                                        ABTI1302
         C = SQRT(C*0.5+0.5)                                            ABTI1303
         S = S/(C+C)                                                    ABTI1304
         LE1 = LE                                                       ABTI1305
         LE = LE1+LE1                                                   ABTI1306
         DO 5 I=1,N,LE                                                  ABTI1307
            I2 = I+I                                                    ABTI1308
            IP2 = I2+LE                                                 ABTI1309
            TI = F(IP2)                                                 ABTI1310
            F(IP2) = F(I2)-TI                                           ABTI1311
            F(I2) = F(I2)+TI                                            ABTI1312
            I2 = I2-1                                                   ABTI1313
            IP2 = IP2-1                                                 ABTI1314
            TR = F(IP2)                                                 ABTI1315
            F(IP2) = F(I2)-TR                                           ABTI1316
            F(I2) = F(I2)+TR                                            ABTI1317
    5    CONTINUE                                                       ABTI1318
         DO 7 J=2,LE1                                                   ABTI1319
            DO 6 I=J,N,LE                                               ABTI1320
               I2 = I+I                                                 ABTI1321
               IP2 = I2+LE                                              ABTI1322
               TR = F(IP2-1)*UR-F(IP2)*UI                               ABTI1323
               TI = F(IP2)*UR+F(IP2-1)*UI                               ABTI1324
               F(IP2) = F(I2)-TI                                        ABTI1325
               F(I2) = F(I2)+TI                                         ABTI1326
               I2 = I2-1                                                ABTI1327
               IP2 = IP2-1                                              ABTI1328
               F(IP2) = F(I2)-TR                                        ABTI1329
               F(I2) = F(I2)+TR                                         ABTI1330
    6       CONTINUE                                                    ABTI1331
            TR = UR*WR-UI*WI                                            ABTI1332
            UI = UI*WR+UR*WI                                            ABTI1333
            UR = TR                                                     ABTI1334
    7    CONTINUE                                                       ABTI1335
    8 CONTINUE                                                          ABTI1336
C                                                                       ABTI1337
C *** FOR J AND K = 0, 1 ... N-1, TRANSFORM THE ARRAY F SO OBTAINED INTOABTI1338
C     SUM F(J+1)*COS(2*PI*K*J/N) AND SUM F(J+1)*SIN(2*PI*K*J/N)         ABTI1339
C     AND MULTIPLY THE RESULTS BY (1-COS(2*PI*K/N)) AND SIN(2*PI*K/N)   ABTI1340
C                                                                       ABTI1341
   10 FNP1 = 4.0*(F(1)-F(2))                                            ABTI1342
      A = 3.1415926/N                                                   ABTI1343
      N2 = N+1                                                          ABTI1344
      N = 2*N                                                           ABTI1345
      IF(N2.EQ.2) GOTO 14                                               ABTI1346
      C = 1.0                                                           ABTI1347
      S = 0.0                                                           ABTI1348
      CA = COS(A)                                                       ABTI1349
      SA = SIN(A)                                                       ABTI1350
      K = N+1                                                           ABTI1351
      DO 11 J=3,N2,2                                                    ABTI1352
         K = K-2                                                        ABTI1353
         D = C                                                          ABTI1354
         C = D*CA-S*SA                                                  ABTI1355
         S = D*SA+S*CA                                                  ABTI1356
         TR1 = F(J)+F(K)                                                ABTI1357
         TI1 = F(J+1)-F(K+1)                                            ABTI1358
         D = F(J)-F(K)                                                  ABTI1359
         E = F(J+1)+F(K+1)                                              ABTI1360
         TR2 = D*S+E*C                                                  ABTI1361
         TI2 = E*S-D*C                                                  ABTI1362
         F(J) = (1.0-C)*(TR1+TR2)                                       ABTI1363
         F(J+1) = S*(TI1+TI2)                                           ABTI1364
         F(K) = (1.0+C)*(TR1-TR2)                                       ABTI1365
         F(K+1) = S*(TI2-TI1)                                           ABTI1366
   11 CONTINUE                                                          ABTI1367
      N2 = N2-1                                                         ABTI1368
      DO 12 J=2,N2                                                      ABTI1369
         J2 = J+J                                                       ABTI1370
         F(J) = F(J2-1)+F(J2)                                           ABTI1371
   12 CONTINUE                                                          ABTI1372
      DO 13 J=2,N2                                                      ABTI1373
         F(N+2-J) = F(J)                                                ABTI1374
   13 CONTINUE                                                          ABTI1375
      N2 = N2+1                                                         ABTI1376
   14 F(N2) = FNP1                                                      ABTI1377
      IF(MSIGN.LT.0) GOTO 16                                            ABTI1378
C                                                                       ABTI1379
C *** NORMALIZATION                                                     ABTI1380
C                                                                       ABTI1381
      C = 0.0                                                           ABTI1382
      A = (A+A)/H                                                       ABTI1383
      DO 15 J=2,N                                                       ABTI1384
         C = C+A                                                        ABTI1385
         F(J) = F(J)/C                                                  ABTI1386
   15 CONTINUE                                                          ABTI1387
   16 F(1) = 0.0                                                        ABTI1388
      RETURN                                                            ABTI1389
   17 WRITE(6,18) H                                                     ABTI1390
   18 FORMAT(' *** INCORRECT STEP SIZE IN <SINTR>, H =',E15.6,' ***')   ABTI1391
      STOP                                                              ABTI1392
      END                                                               ABTI1393
      SUBROUTINE RCFFI(AR,AI,MSIGN,H)                                   ABTI1394
C   ******************************************************************* ABTI1395
C   *                                                                 * ABTI1396
C   * USING A RADIX-TWO FAST-FOURIER-TRANSFORM TECHNIQUE, RCFFI       * ABTI1397
C   * COMPUTES THE FOURIER INTEGRAL TRANSFORM OF A REAL FUNCTION      * ABTI1398
C   * F(X) OR THE INVERSE FOURIER TRANSFORM OF A COMPLEX FUNCTION     * ABTI1399
C   * SUCH THAT G(-Y) = CONJ(G(Y)).                                   * ABTI1400
C   * (ADAPTED FROM CFFT ROUTINE, CERN LIBRARY MEMBER D704)           * ABTI1401
C   *                                                                 * ABTI1402
C   * G(Y) = INTEGRAL F(X)*CEXP(-I*X*Y) DX    (MSIGN < 0)             * ABTI1403
C   *                                                                 * ABTI1404
C   * F(X) = 1/(2*PI) INTEGRAL G(Y)*CEXP(+I*Y*X) DY    (MSIGN > 0)    * ABTI1405
C   *                                                                 * ABTI1406
C   * F(X) IS TABULATED ON THE MESHES OF POINTS DEFINED BY            * ABTI1407
C   * XJ = J*HX AND -XJ = -J*HX  ,  J = 0, +1, ..., N-2, N-1          * ABTI1408
C   * AR(J+1) CONTAINS F(+XJ) AND AI(J+1) CONTAINS F(-XJ)             * ABTI1409
C   * (INPUT WHEN MSIGN < 0 , OUTPUT WHEN MSIGN >0)                   * ABTI1410
C   * AR(1) MAY BE DIFFERENT FROM AI(1)                               * ABTI1411
C   *                                                                 * ABTI1412
C   * G(Y) IS TABULATED ON THE MESH OF POINTS DEFINED BY              * ABTI1413
C   * YK = K*HY  , K = 0, 1, ..., N-2, N-1                            * ABTI1414
C   * AR(J+1) CONTAINS REAL(G(YK)) AND AI(J+1) CONTAINS AIMAG(G(YK))  * ABTI1415
C   * (OUTPUT WHEN MSIGN > 0 , INPUT WHEN MSIGN < 0)                  * ABTI1416
C   * REMARK :  G(-YK) = CONJ(G(+YK))                                 * ABTI1417
C   *                                                                 * ABTI1418
C   * N = 2**IABS(MSIGN)  (MSIGN IS AN INPUT)                         * ABTI1419
C   *                                                                 * ABTI1420
C   * THE STEP SIZES SATISFY                                          * ABTI1421
C   * HY = (2*PI)/(N*HX)   OR    HX = (2*PI)/(N*HY)                   * ABTI1422
C   * H IS AN INPUT (H = HX IF MSIGN < 0 , H = HY IF MSIGN > 0)       * ABTI1423
C   * H MUST BE POSITIVE                                              * ABTI1424
C   *                                                                 * ABTI1425
C   * AR AND AI ARE TWO REAL ARRAYS WITH DIMENSION N (INPUT AND       * ABTI1426
C   * OUTPUT)                                                         * ABTI1427
C   *                                                                 * ABTI1428
C   ******************************************************************* ABTI1429
      DIMENSION AR(*),AI(*)                                             ABTI1430
      IF(MSIGN.EQ.0) RETURN                                             ABTI1431
      IF(H.LE.0.0) GOTO 16                                              ABTI1432
C                                                                       ABTI1433
C *** INITIALIZATION                                                    ABTI1434
C                                                                       ABTI1435
      M = IABS(MSIGN)                                                   ABTI1436
      N = 2**M                                                          ABTI1437
      IF(MSIGN.GT.0) GOTO 3                                             ABTI1438
      AS = AR(1)-AI(1)                                                  ABTI1439
      AR(1) = 0.5*(AR(1)+AI(1))                                         ABTI1440
      DO 1 I=2,N                                                        ABTI1441
         AR(I) = AR(I)+AI(N-I+2)                                        ABTI1442
    1 CONTINUE                                                          ABTI1443
      DO 2 I=1,N                                                        ABTI1444
         AI(I) = 0.0                                                    ABTI1445
    2 CONTINUE                                                          ABTI1446
      GOTO 4                                                            ABTI1447
    3 AR(1) = 0.5*AR(1)                                                 ABTI1448
C                                                                       ABTI1449
C *** DISCRETE FAST-FOURIER TRANSFORM                                   ABTI1450
C                                                                       ABTI1451
    4 NV2 = N/2                                                         ABTI1452
      NM1 = N-1                                                         ABTI1453
      J = 1                                                             ABTI1454
      DO 7 I=1,NM1                                                      ABTI1455
         IF(I.LT.J) THEN                                                ABTI1456
            TR = AR(J)                                                  ABTI1457
            AR(J) = AR(I)                                               ABTI1458
            AR(I) = TR                                                  ABTI1459
            TI = AI(J)                                                  ABTI1460
            AI(J) = AI(I)                                               ABTI1461
            AI(I) = TI                                                  ABTI1462
         ENDIF                                                          ABTI1463
         K = NV2                                                        ABTI1464
    5    IF(K.GE.J) GOTO 6                                              ABTI1465
         J = J-K                                                        ABTI1466
         K = K/2                                                        ABTI1467
         GOTO 5                                                         ABTI1468
    6    J = J+K                                                        ABTI1469
    7 CONTINUE                                                          ABTI1470
      DO 8 I=1,N,2                                                      ABTI1471
         TR = AR(I+1)                                                   ABTI1472
         AR(I+1) = AR(I)-TR                                             ABTI1473
         AR(I) = AR(I)+TR                                               ABTI1474
         TI = AI(I+1)                                                   ABTI1475
         AI(I+1) = AI(I)-TI                                             ABTI1476
         AI(I) = AI(I)+TI                                               ABTI1477
    8 CONTINUE                                                          ABTI1478
      IF(M.EQ.1) RETURN                                                 ABTI1479
      C = 0.0                                                           ABTI1480
      S = FLOAT(ISIGN(1,MSIGN))                                         ABTI1481
      LE = 2                                                            ABTI1482
      DO 12 L=2,M                                                       ABTI1483
         WR = C                                                         ABTI1484
         UR = WR                                                        ABTI1485
         WI = S                                                         ABTI1486
         UI = WI                                                        ABTI1487
         C = SQRT(C*0.5+0.5)                                            ABTI1488
         S = WI/(C+C)                                                   ABTI1489
         LE1 = LE                                                       ABTI1490
         LE = LE1+LE1                                                   ABTI1491
         DO 9 I=1,N,LE                                                  ABTI1492
            IP = I+LE1                                                  ABTI1493
            TR = AR(IP)                                                 ABTI1494
            AR(IP) = AR(I)-TR                                           ABTI1495
            AR(I) = AR(I)+TR                                            ABTI1496
            TI = AI(IP)                                                 ABTI1497
            AI(IP) = AI(I)-TI                                           ABTI1498
            AI(I) = AI(I)+TI                                            ABTI1499
    9    CONTINUE                                                       ABTI1500
         DO 11 J=2,LE1                                                  ABTI1501
            DO 10 I=J,N,LE                                              ABTI1502
               IP = I+LE1                                               ABTI1503
               TR = AR(IP)*UR-AI(IP)*UI                                 ABTI1504
               TI = AR(IP)*UI+AI(IP)*UR                                 ABTI1505
               AR(IP) = AR(I)-TR                                        ABTI1506
               AR(I) = AR(I)+TR                                         ABTI1507
               AI(IP) = AI(I)-TI                                        ABTI1508
               AI(I) = AI(I)+TI                                         ABTI1509
   10       CONTINUE                                                    ABTI1510
            TR = UR                                                     ABTI1511
            UR = TR*WR-UI*WI                                            ABTI1512
            UI = TR*WI+UI*WR                                            ABTI1513
   11    CONTINUE                                                       ABTI1514
   12 CONTINUE                                                          ABTI1515
C                                                                       ABTI1516
C *** CORRECTION OF THE DISCRETE FFT, USING A TRAPEZOIDAL APPROXIMATION ABTI1517
C *** FOR THE VARIATIONS OF THE INPUT FUNCTION                          ABTI1518
C                                                                       ABTI1519
      C = H                                                             ABTI1520
      IF(MSIGN.LT.0) GOTO 14                                            ABTI1521
      C = C/3.1415926                                                   ABTI1522
      AI(1) = AR(1)                                                     ABTI1523
      DO 13 I=2,N                                                       ABTI1524
         AI(I) = AR(N-I+2)                                              ABTI1525
   13 CONTINUE                                                          ABTI1526
   14 AR(1) = C*AR(1)                                                   ABTI1527
      AI(1) = C*AI(1)                                                   ABTI1528
      A = 3.1415926/N                                                   ABTI1529
      WR = COS(A)                                                       ABTI1530
      WI = SIN(A)                                                       ABTI1531
      T = SQRT(C)                                                       ABTI1532
      A = A/T                                                           ABTI1533
      C = 0.0                                                           ABTI1534
      UR = 1.0                                                          ABTI1535
      UI = 0.0                                                          ABTI1536
      DO 15 I=2,N                                                       ABTI1537
         C = C+A                                                        ABTI1538
         TR = UR                                                        ABTI1539
         UR = WR*TR-WI*UI                                               ABTI1540
         UI = WI*TR+WR*UI                                               ABTI1541
         TI = UI/C                                                      ABTI1542
         TR = TI**2                                                     ABTI1543
         AR(I) = AR(I)*TR                                               ABTI1544
         AI(I) = AI(I)*TR                                               ABTI1545
         IF(MSIGN.GT.0) GOTO 15                                         ABTI1546
         IF(AS.NE.0.0) AI(I) = AI(I)-AS*(T-UR*TI)/(C+C)                 ABTI1547
   15 CONTINUE                                                          ABTI1548
      RETURN                                                            ABTI1549
   16 WRITE(6,17) H                                                     ABTI1550
   17 FORMAT(' *** NEGATIVE STEP SIZE IN <RCFFI>, H =',E13.5,' ***')    ABTI1551
      STOP                                                              ABTI1552
      END                                                               ABTI1553