program eels

! ******************************************************************
! *                                                                *
! * compute the classical eels spectrum of an arbitrary plane-     *
! * statified medium made from isotropic materials in specular     *
! * geometry using the dielectric theory of eels.                  *
! *                                                                *
! ******************************************************************

  implicit none

  double precision :: e0, theta, phia, phib, wmin, wmax, dw
  integer :: i, j, jos, k, l, layers, neps, nper, nw
  logical :: user
  character (len = 72) :: comment(2)
  character (len = 10) :: contrl, mode

  double precision, allocatable :: thick(:), epsinf(:), osc(:, :)
  integer, allocatable :: nos(:)
  character (len = 10), allocatable :: name(:)
  double precision, allocatable :: wn_array(:), f(:)

  integer :: old_size_1, old_size_2
  double precision, allocatable :: tmp_osc(:, :)
  integer :: ioStatus

!  integer, parameter :: name_length = 10
! *** read spectrometer parameters

      call FCAT_eels_all(1)
  call change_working_dir()
      call FCAT_eels_all(2)
  open(unit = 11, file = 'eelsin')
! impact energy (ev)
      call FCAT_eels_all(3)
  read(11, *) e0
! incidence angle (%)
      call FCAT_eels_all(4)
  read(11, *) theta
! angular apertures of the elliptic detector (%)
      call FCAT_eels_all(5)
  read(11, *) phia
      call FCAT_eels_all(6)
  read(11, *) phib
! energy-loss interval and step size (cm**-1)
      call FCAT_eels_all(7)
  read(11, *) wmin
      call FCAT_eels_all(8)
  read(11, *) wmax
      call FCAT_eels_all(9)
  read(11, *) dw
! comment lines
      call FCAT_eels_all(10)
  read(11, '(a72)') (comment(k), k = 1, 2)

      call FCAT_eels_all(11)
  write(*,*) 'program eels (September 2020)'
      call FCAT_eels_all(12)
  write(*,'(a, f6.2, a, f5.1, a, f5.2, a, f5.2, a)') &
    ' e0 = ', e0, ' eV, theta = ', theta, '°, phia = ', &
    phia, '°, phib = ', phib, '°'
      call FCAT_eels_all(13)
  write(*,'(a, g11.4, a, g11.4, a, g11.4, a)') &
    ' energy losses from', wmin, ' to', wmax, ', step = ', dw, ' cm**-1'
      call FCAT_eels_all(14)
  write(*,*) comment(1)
      call FCAT_eels_all(15)
  write(*,*) comment(2)

      call FCAT_eels_all(16)
  if (phia <= 0.0d0 .or. phib <= 0.0d0) then
      call FCAT_eels_all(17)
      call FCAT_eels_all_rep()
    stop '*** wrong input ***'
      call FCAT_eels_all(18)
  endif
      call FCAT_eels_all(19)
  if (e0 <= 0.0d0 .or. theta + phia >= 90.0d0) then
      call FCAT_eels_all(20)
      call FCAT_eels_all_rep()
    stop '*** bad input ***'
      call FCAT_eels_all(21)
  endif

! *** read target specifications

      call FCAT_eels_all(22)
  read(11, *) layers, nper, mode
      call FCAT_eels_all(23)
  user = layers == 0
      call FCAT_eels_all(24)
  if (.not. user) then
      call FCAT_eels_all(25)
    neps = layers
      call FCAT_eels_all(26)
    if (nper == -1) then
      call FCAT_eels_all(27)
      neps = layers + 1
      call FCAT_eels_all(28)
      nper = 1
      call FCAT_eels_all(29)
      write(*,*) 'the substrate is a anisotropic uniaxial material'
      call FCAT_eels_all(30)
    endif
      call FCAT_eels_all(31)
    if (layers < 0 .or. nper < 1 .or. nper > layers) then
      call FCAT_eels_all(32)
      call FCAT_eels_all_rep()
      stop '*** invalid target specifications ***'
      call FCAT_eels_all(33)
    endif
      call FCAT_eels_all(34)
    write(6, 101) layers, nper
      call FCAT_eels_all(35)
    jos = 0
      call FCAT_eels_all(36)
    allocate (name(neps), thick(neps), epsinf(neps), nos(neps))
      call FCAT_eels_all(37)
    do l = 1, neps
      call FCAT_eels_all(38)
      if (l <= layers) then
      call FCAT_eels_all(39)
        read(11, 102) name(l), thick(l)
      call FCAT_eels_all(40)
      endif
      call FCAT_eels_all(41)
      read(11, *) epsinf(l), nos(l)
      call FCAT_eels_all(42)
      write(6, 103)
      call FCAT_eels_all(43)
      if (nos(l) <= 0) then
      call FCAT_eels_all(44)
        if (l <= layers) write(6, 104) l, name(l), thick(l), epsinf(l)
      call FCAT_eels_all(45)
        if (l  > layers) write(6, 105) epsinf(l)
      else
      call FCAT_eels_all(46)
      call FCAT_eels_all(47)
        do j = 1, nos(l)
      call FCAT_eels_all(48)
          if (.not. allocated(osc)) then
      call FCAT_eels_all(49)
            allocate(osc(3, nos(l)))
          else if (j == 1) then
 !           call extend3(osc, nos(j))
      call FCAT_eels_all(50)
      call FCAT_eels_all(51)
            old_size_1 = size(osc, 1)
      call FCAT_eels_all(52)
            old_size_2 = size(osc, 2)
      call FCAT_eels_all(53)
            allocate(tmp_osc(old_size_1, old_size_2 + nos(l)))
      call FCAT_eels_all(54)
            tmp_osc(1:old_size_1, 1:old_size_2) = osc
      call FCAT_eels_all(55)
            deallocate(osc)
      call FCAT_eels_all(56)
            call move_alloc(tmp_osc, osc)
      call FCAT_eels_all(57)
          endif
      call FCAT_eels_all(58)
          jos = jos + 1
      call FCAT_eels_all(59)
          read(11, *) (osc(k, jos), k = 1, 3)
      call FCAT_eels_all(60)
          if ((j == nos(l) / 2 + 1) .and. (nos(l) > 1)) then
      call FCAT_eels_all(61)
            write(6, 106)
      call FCAT_eels_all(62)
            write(6, 107)
      call FCAT_eels_all(63)
          endif
      call FCAT_eels_all(64)
          if (j == 1) then
      call FCAT_eels_all(65)
            if (l <= layers) then
      call FCAT_eels_all(66)
              write(6, 104) l, name(l), thick(l), epsinf(l), (osc(i, jos), i = 1, 3)
            else
      call FCAT_eels_all(67)
      call FCAT_eels_all(68)
              write(6, 105) epsinf(l), (osc(i, jos), i = 1, 3)
      call FCAT_eels_all(69)
            endif
          else
      call FCAT_eels_all(70)
      call FCAT_eels_all(71)
            write(6, 108) (osc(i, jos), i = 1, 3)
      call FCAT_eels_all(72)
          endif
      call FCAT_eels_all(73)
        enddo
      call FCAT_eels_all(74)
      endif
      call FCAT_eels_all(75)
    enddo
      call FCAT_eels_all(76)
    write(*,*)
      call FCAT_eels_all(77)
    read(11, 102, IOSTAT = ioStatus) contrl
      call FCAT_eels_all(78)
    if (ioStatus /= 0) then
      call FCAT_eels_all(79)
      contrl = ''
      call FCAT_eels_all(80)
    endif
      call FCAT_eels_all(81)
  endif

      call FCAT_eels_all(82)
  close (unit = 11)

      call FCAT_eels_all(83)
  nw = 1 + int((wmax - wmin) / dw)
      call FCAT_eels_all(84)
  allocate (wn_array(nw), f(nw))

      call FCAT_eels_all(85)
  call doeels(e0, theta, phia, phib, wmin, wmax, dw, comment, size(comment),   &
              layers, neps, nper, name, size(name), thick, epsinf, nos, osc, size(osc, 2),&
              contrl, mode, wn_array, f, size(wn_array))

      call FCAT_eels_all(86)
  open (unit = 12, file = 'eelsou')
      call FCAT_eels_all(87)
  write (12, 207) e0, theta, phia, phib, comment(1)
      call FCAT_eels_all(88)
  do i = 1, nw
      call FCAT_eels_all(89)
    write (12, 211) wn_array(i), f(i)
      call FCAT_eels_all(90)
  enddo
      call FCAT_eels_all(91)
  close (unit = 12)

      call FCAT_eels_all(92)
      call FCAT_eels_all_rep()
  stop

      call FCAT_eels_all(93)
101 format(i3, ' layer(s), nper = ', i2//'   l', 2x, 'material', 7x,  &
      'thickness', 5x, 'epsinf', 4x, 'wto , wp', 5x, 'q', 7x, 'gam/wto')
      call FCAT_eels_all(94)
102 format(a10, d15.5)
      call FCAT_eels_all(95)
103 format(1x, 72('-'))
      call FCAT_eels_all(96)
104 format(1x, i3, 2x, a10, g15.3, f10.4, f12.4, f10.4, f9.4)
      call FCAT_eels_all(97)
105 format(31x, f10.4, f12.4, f10.4, f9.4)
      call FCAT_eels_all(98)
106 format(45x, 'wlo , wp', 5x, 'q', 7x, 'gam/wlo')
      call FCAT_eels_all(99)
107 format(45x, 28('-'))
      call FCAT_eels_all(100)
108 format(41x, f12.4, f10.4, f9.4)
      call FCAT_eels_all(101)
207 format('e0 =', f6.2, ' theta =', f5.1, ' phia =', f5.2, ' phib =', f5.2 / a72)
      call FCAT_eels_all(102)
211 format(2e15.7)
      call FCAT_eels_all_rep()
end program eels

double precision function qrat(x, alpha, beta, c1, c2)

  implicit none

  double precision, intent(in) :: x, alpha, beta, c1, c2

      call FCAT_eels_all(103)
  qrat = (1.0d0 + x * (beta + c1 * x)) / ((1.0d0 + x * (beta + c2 * x)) * (1.0d0 + alpha * x)**2)

      call FCAT_eels_all(104)
  return
      call FCAT_eels_all_rep()
end function qrat

double precision function usurlo(dq, wn)

! ******************************************************************
! *                                                                *
! * user-supplied dielectric surface loss function aimag(g(dq, wn)) *
! * input arguments :                                              *
! *    dq : modulus of the two-dimensional surface wave vector     *
! *         (angstroem**-1)                                        *
! *    wn : frequency (cm**-1)                                     *
! *                                                                *
! ******************************************************************

  implicit none

  double precision, intent(in) :: dq
  double precision, intent(in) :: wn

      call FCAT_eels_all(105)
  usurlo = 1.0d0
      call FCAT_eels_all(106)
  return
end function usurlo
double precision function surlos(dk, eps, thick, layers, nper)

! ******************************************************************
! *                                                                *
! * eels surface loss function for an arbitrary multilayered target*
! *                                                                *
! ******************************************************************

  implicit none

  double precision, intent(in) :: dk
  double complex, intent(in) :: eps(:)
  double precision, intent(in) :: thick(:)
  integer, intent(in) :: layers, nper

  integer :: lmax
  logical :: static, zero, skip
  integer :: lstart, n
  double precision, allocatable :: arg(:)
  double precision :: argmin, argmax, cn, cnm1, epsmac, sn, snm1, tn
  double complex :: a, b, csi, pnm2, pnm1, pn, pp, qnm2, qnm1, qn, qp, z

  common / mulayr / argmin, argmax, epsmac

  zero(z) = (dble(z) == 0.0d0) .and. (dimag(z) == 0.0d0)

!  write (*,*) 'surlos:'
!  write (*,*) 'thick: ', size(thick)
!  write (*,*) 'eps: ', size(eps)

      call FCAT_eels_all(107)
  lmax = size(eps)
      call FCAT_eels_all(108)
  allocate (arg(lmax))
      call FCAT_eels_all(109)
  lstart = layers - nper + 1
      call FCAT_eels_all(110)
  static = .true.
      call FCAT_eels_all(111)
  skip = .false.

      call FCAT_eels_all(112)
  do n = 1, layers
      call FCAT_eels_all(113)
    arg(n) = dk * thick(n)
      call FCAT_eels_all(114)
    if (arg(n) > argmax .or. zero(eps(n))) then
      call FCAT_eels_all(115)
      csi  = eps(n)
      call FCAT_eels_all(116)
      skip = .true.
      call FCAT_eels_all(117)
      exit
      call FCAT_eels_all(118)
    endif
      call FCAT_eels_all(119)
    static = .not. (n >= lstart .and. arg(n) > argmin)
      call FCAT_eels_all(120)
  enddo

      call FCAT_eels_all(121)
  if (.not. skip) then
! *** periodic continued fraction, period = nper

      call FCAT_eels_all(122)
    do ! Do loop is only for the option to skip out of it.
      call FCAT_eels_all(123)
      if (nper <= 1) then
      call FCAT_eels_all(124)
        csi = eps(layers)
      else
      call FCAT_eels_all(125)
      call FCAT_eels_all(126)
        if (static) then
      call FCAT_eels_all(127)
          pn = 0.0d0
      call FCAT_eels_all(128)
          qn = 0.0d0
      call FCAT_eels_all(129)
          do n = lstart, layers
      call FCAT_eels_all(130)
            pn = pn + thick(n) * eps(n)
      call FCAT_eels_all(131)
            qn = qn + thick(n) / eps(n)
      call FCAT_eels_all(132)
          enddo
      call FCAT_eels_all(133)
          if (zero(qn)) then
      call FCAT_eels_all(134)
            n = lstart
      call FCAT_eels_all(135)
            if (n <= 1) then
      call FCAT_eels_all(136)
              surlos = 0.0d0
      call FCAT_eels_all(137)
              return
      call FCAT_eels_all(138)
            endif
      call FCAT_eels_all(139)
            n = n - 1
      call FCAT_eels_all(140)
            csi = eps(n) / dtanh(arg(n))
      call FCAT_eels_all(141)
            exit
      call FCAT_eels_all(142)
          endif ! zero(qn)
      call FCAT_eels_all(143)
          csi = cdsqrt(pn / qn)
      call FCAT_eels_all(144)
          if ((dimag(csi) < 0.0d0) .and. (dble(qn) < 0.0d0)) then
      call FCAT_eels_all(145)
            csi = -csi
      call FCAT_eels_all(146)
          endif
        else ! static
      call FCAT_eels_all(147)
      call FCAT_eels_all(148)
          cn = dcosh(arg(lstart))
      call FCAT_eels_all(149)
          sn = dsinh(arg(lstart))
      call FCAT_eels_all(150)
          pnm1 = 1.0d0
      call FCAT_eels_all(151)
          pn = cn
      call FCAT_eels_all(152)
          pp = eps(lstart) * sn
      call FCAT_eels_all(153)
          qnm1 = 0.0d0
      call FCAT_eels_all(154)
          qn = sn / eps(lstart)
      call FCAT_eels_all(155)
          qp = pn
      call FCAT_eels_all(156)
          do  n = lstart + 1, layers
      call FCAT_eels_all(157)
            cnm1 = cn
      call FCAT_eels_all(158)
            snm1 = sn
      call FCAT_eels_all(159)
            cn = dcosh(arg(n))
      call FCAT_eels_all(160)
            sn = dsinh(arg(n))
      call FCAT_eels_all(161)
            a = eps(n) * sn
      call FCAT_eels_all(162)
            pp = cn * pp + a * pn
      call FCAT_eels_all(163)
            qp = cn * qp + a * qn
      call FCAT_eels_all(164)
            b = (eps(n - 1) / eps(n)) * (sn / snm1)
      call FCAT_eels_all(165)
            a = cnm1 * b + cn
      call FCAT_eels_all(166)
            pnm2 = pnm1
      call FCAT_eels_all(167)
            pnm1 = pn
      call FCAT_eels_all(168)
            qnm2 = qnm1
      call FCAT_eels_all(169)
            qnm1 = qn
      call FCAT_eels_all(170)
            pn = a * pnm1 - b * pnm2
      call FCAT_eels_all(171)
            qn = a * qnm1 - b * qnm2
      call FCAT_eels_all(172)
          enddo
      call FCAT_eels_all(173)
          if (zero(qn)) then
      call FCAT_eels_all(174)
            a = qp - pn
      call FCAT_eels_all(175)
            if (zero(a)) then
      call FCAT_eels_all(176)
              n = lstart
      call FCAT_eels_all(177)
              if (n <= 1) then
      call FCAT_eels_all(178)
                surlos = 0.0d0
      call FCAT_eels_all(179)
                return
      call FCAT_eels_all(180)
              endif
      call FCAT_eels_all(181)
              n = n - 1
      call FCAT_eels_all(182)
              csi = eps(n) / dtanh(arg(n))
      call FCAT_eels_all(183)
              exit
      call FCAT_eels_all(184)
            endif
      call FCAT_eels_all(185)
            csi = pp / a
          else
      call FCAT_eels_all(186)
      call FCAT_eels_all(187)
            a = 0.5d0 * (pn - qp) / qn
      call FCAT_eels_all(188)
            b = cdsqrt(a**2 + pp / qn)
      call FCAT_eels_all(189)
            pn = a - pn / qn
      call FCAT_eels_all(190)
            if (cdabs(pn + b) > cdabs(pn - b)) then
      call FCAT_eels_all(191)
              b = -b
      call FCAT_eels_all(192)
            endif
      call FCAT_eels_all(193)
            csi = a + b
      call FCAT_eels_all(194)
          endif ! zero(qn)
      call FCAT_eels_all(195)
        endif ! static
! *** small-dk limit of the periodic tail

      call FCAT_eels_all(196)
      endif ! nper <= 1

      call FCAT_eels_all(197)
      n = lstart

      call FCAT_eels_all(198)
      exit
      call FCAT_eels_all(199)
    enddo
      call FCAT_eels_all(200)
  endif ! .not. skip

! *** backward algorithm

      call FCAT_eels_all(201)
  do
      call FCAT_eels_all(202)
    n = n - 1
      call FCAT_eels_all(203)
    if (n <= 0) then
      call FCAT_eels_all(204)
      a = csi + 1.0d0
      call FCAT_eels_all(205)
      if (zero(a)) then
      call FCAT_eels_all(206)
        surlos = 2 / epsmac
      else
      call FCAT_eels_all(207)
      call FCAT_eels_all(208)
        surlos = dimag(-2 / a)
      call FCAT_eels_all(209)
      endif
      call FCAT_eels_all(210)
      return
      call FCAT_eels_all(211)
    endif ! n <= 0

      call FCAT_eels_all(212)
    if (arg(n) /= 0.0d0) then
      call FCAT_eels_all(213)
      tn = dtanh(arg(n))
      call FCAT_eels_all(214)
      b = eps(n) + csi * tn
      call FCAT_eels_all(215)
      if (zero(b)) then
      call FCAT_eels_all(216)
        surlos = 0.0d0
      call FCAT_eels_all(217)
        return
      call FCAT_eels_all(218)
      endif
      call FCAT_eels_all(219)
      csi = eps(n) * (csi + tn * eps(n)) / b
      call FCAT_eels_all(220)
    endif
      call FCAT_eels_all(221)
  enddo
end function surlos
double precision function phint(phi, a, u)

! ******************************************************************
! *                                                                *
! * evaluate the integral from zero to phi of                      *
! *                                                                *
! *                 u                 2                            *
! *  ( ----------------------------- )  dphi                       *
! *                          2     2                               *
! *    (1 - a * u * cos(phi))  +  u                                *
! *                                                                *
! * for 0 <= phi <= pi , u >= 0 and a >= 0                         *
! *                                                                *
! ******************************************************************

  implicit none

  double precision, intent(in) :: phi
  double precision, intent(in) :: a
  double precision, intent(in) :: u

  double precision :: ai, ar, bi, br, c, cpr, d, e, esr, pi, qr, ri, rm, root
  double precision :: rp, rr, s, spr, tm, tp, u2, x, zeta, zetai, zetar, zr

      call FCAT_eels_all(222)
  pi = 3.141592653589793238d0
      call FCAT_eels_all(223)
  c = dcos(phi)
      call FCAT_eels_all(224)
  s = dsin(phi)
      call FCAT_eels_all(225)
  u2 = u**2
      call FCAT_eels_all(226)
  e = a*u
      call FCAT_eels_all(227)
  if (u < 1.0d0 .and. e < 1.0d-02 * (1.0d0 + u2)) then
      call FCAT_eels_all(228)
    zr = 1.0d0 + u2
      call FCAT_eels_all(229)
    esr = e / zr
      call FCAT_eels_all(230)
    phint = u2 / zr**2 * ((( (4.0d0 / 3.0d0) * (2.0d0 + c**2) * s * (5.0d0 - 3 * u2) *  &
            esr + (phi + c * s) * (5.0d0 - u2)) * esr + 4 * s) * esr + phi)
  else
      call FCAT_eels_all(231)
      call FCAT_eels_all(232)
    rm = dsqrt((1.0d0 - e)**2 + u2)
      call FCAT_eels_all(233)
    tm = 0.5d0 * datan2(u, 1.0d0 - e)
      call FCAT_eels_all(234)
    rp = dsqrt((1.0d0 + e)**2 + u2)
      call FCAT_eels_all(235)
    tp = 0.5d0 * datan2(u, 1.0d0 + e)
      call FCAT_eels_all(236)
    root = dsqrt(rm * rp)
      call FCAT_eels_all(237)
    cpr = dcos(tm + tp)
      call FCAT_eels_all(238)
    spr = dsin(tm + tp)
      call FCAT_eels_all(239)
    x = 0.0d0       ! ensure initialization
      call FCAT_eels_all(240)
    if (c >= 0.0d0) then
      call FCAT_eels_all(241)
      x = s / (1.0d0 + c)
    elseif (dabs(s) > 1.0d-07) then
      call FCAT_eels_all(242)
      call FCAT_eels_all(243)
      x = (1.0d0 - c) / s
      call FCAT_eels_all(244)
    endif
      call FCAT_eels_all(245)
    if ((c >= 0.0d0) .or. (dabs(s) > 1.0d-07)) then
      call FCAT_eels_all(246)
      zeta = dsqrt(rm / rp)
      call FCAT_eels_all(247)
      zetar = -zeta * dsin(tm - tp)
      call FCAT_eels_all(248)
      zetai =  zeta * dcos(tm - tp)
      call FCAT_eels_all(249)
      br = 0.5d0 * dlog(((zetar + x)**2 + zetai**2) / ((zetar - x)**2 + zetai**2))
      call FCAT_eels_all(250)
      bi = datan2(zetai, zetar + x) - datan2(zetai, zetar - x)
      call FCAT_eels_all(251)
      rr = -(br * spr - bi * cpr) / root
      call FCAT_eels_all(252)
      ri = -(bi * spr + br * cpr) / root
      call FCAT_eels_all(253)
      d = e * s / ((1.0d0 - e * c)**2 + u2)
      call FCAT_eels_all(254)
      ar = d * (1.0d0 - e * c) - rr + u * ri
      call FCAT_eels_all(255)
      ai = -d * u - ri - u * rr
    else
      call FCAT_eels_all(256)
      call FCAT_eels_all(257)
      rr = -pi / root * cpr
      call FCAT_eels_all(258)
      ri =  pi / root * spr
      call FCAT_eels_all(259)
      ar = -rr + u * ri
      call FCAT_eels_all(260)
      ai = -ri - u * rr
      call FCAT_eels_all(261)
    endif
      call FCAT_eels_all(262)
    qr = (ar * (cpr - spr) * (cpr + spr) + 2 * ai * cpr * spr) / (rm * rp)
      call FCAT_eels_all(263)
    phint = 0.5d0 * (ri / u - qr)
      call FCAT_eels_all(264)
  endif
      call FCAT_eels_all(265)
  return
end function phint
double precision function fint1(u, eps, thick, layers, nper, eps_size)

! ******************************************************************
! *                                                                *
! * integration over the azimutal angle from 0.0 to pi             *
! *                                                                *
! ******************************************************************

  implicit none

  double precision, intent(in) :: u
  integer, intent(in) :: layers, nper, eps_size
  double complex, intent(in) :: eps(eps_size)
  double precision, intent(in) :: thick(eps_size)

  logical :: rational, user
  double precision :: acoef, bcoef, ccoef, cospsi, den, dif, dlimf, e, elleps
  double precision :: pi, rom, rop, ru, sinpsi, sum, um, t
  double precision :: tanpsi, wn, u2

  interface
    double precision function usurlo(dq, wn)
      double precision, intent(in) :: dq
      double precision, intent(in) :: wn
    end function usurlo
    double precision function surlos(dk, eps, thick, layers, nper)
      double precision, intent(in) :: dk
      double complex, intent(in) :: eps(:)
      double precision, intent(in) :: thick(:)
      integer, intent(in) :: layers, nper
    end function surlos
  end interface  

  common / param / acoef, bcoef, ccoef, elleps, cospsi, sinpsi, tanpsi,  &
      ru, um, dlimf, wn, user, rational

  data pi / 3.141592653589793238d0 /

!  write (*,*) 'fint1:'
!  write (*,*) 'thick: ', size(thick)
!  write (*,*) 'eps: ', size(eps)

      call FCAT_eels_all(266)
  if (u == 0.0d0) then
      call FCAT_eels_all(267)
    fint1 = 0.0d0
      call FCAT_eels_all(268)
    return
      call FCAT_eels_all(269)
  endif
      call FCAT_eels_all(270)
  e = tanpsi * u
      call FCAT_eels_all(271)
  u2 = u**2
      call FCAT_eels_all(272)
  rom = (1.0d0 - e)**2 + u2
      call FCAT_eels_all(273)
  rop = (1.0d0 + e)**2 + u2
      call FCAT_eels_all(274)
  sum = rop + rom
      call FCAT_eels_all(275)
  rom = dsqrt(rom)
      call FCAT_eels_all(276)
  rop = dsqrt(rop)
      call FCAT_eels_all(277)
  dif = rop - rom
      call FCAT_eels_all(278)
  den = dsqrt((2.0d0 - dif) * (2.0d0 + dif)) * rop * rom
      call FCAT_eels_all(279)
  fint1 = pi * u2 * (4 * sum - dif**2 * (sum - rop * rom)) / den**3
      call FCAT_eels_all(280)
  if (rational) then
      call FCAT_eels_all(281)
    return
      call FCAT_eels_all(282)
  endif
      call FCAT_eels_all(283)
  if (user) then
      call FCAT_eels_all(284)
    fint1 = fint1 * usurlo(ru * u, wn)
  else
      call FCAT_eels_all(285)
      call FCAT_eels_all(286)
    fint1 = fint1 * surlos(ru * u, eps, thick, layers, nper)
      call FCAT_eels_all(287)
    if (dlimf > 0.0d0) then
      call FCAT_eels_all(288)
      t = ru * u * dlimf
      call FCAT_eels_all(289)
      fint1 = fint1 * (1.d0 + t * dlog(t / (t + 0.26d0)))**2 / (1.d0 + 1.40d0 * t)
      call FCAT_eels_all(290)
    endif
      call FCAT_eels_all(291)
  endif
      call FCAT_eels_all(292)
  return
end function fint1
double precision function fint2(u, eps, thick, layers, nper, eps_size)

! ******************************************************************
! *                                                                *
! * integration over the azimutal angle from 0.0 to phi < pi       *
! *                                                                *
! ******************************************************************

  implicit none

  double precision, intent(in) :: u
  integer, intent(in) :: layers, nper, eps_size
  double complex, intent(in) :: eps(eps_size)
  double precision, intent(in) :: thick(eps_size)

  logical :: rational, user
  double precision :: a, arg, b, b2, c, ccoef, cospsi, dlimf, elleps, phi
  double precision :: phint, ru, sinpsi, um, t, tanpsi, wn, x

  interface
    double precision function usurlo(dq, wn)
      double precision, intent(in) :: dq
      double precision, intent(in) :: wn
    end function usurlo
    double precision function surlos(dk, eps, thick, layers, nper)
      double precision, intent(in) :: dk
      double complex, intent(in) :: eps(:)
      double precision, intent(in) :: thick(:)
      integer, intent(in) :: layers, nper
    end function surlos
  end interface  

  common / param / a, b, ccoef, elleps, cospsi, sinpsi, tanpsi, &
      ru, um, dlimf, wn, user, rational

!  write (*,*) 'fint2:'
!  write (*,*) 'thick: ', size(thick)
!  write (*,*) 'eps: ', size(eps)

      call FCAT_eels_all(293)
  if (u == 0.0d0) then
      call FCAT_eels_all(294)
    fint2 = 0.0d0
      call FCAT_eels_all(295)
    return
      call FCAT_eels_all(296)
  endif
      call FCAT_eels_all(297)
  b2 = b**2
      call FCAT_eels_all(298)
  c = (1.0d0 - elleps) * (cospsi * u)**2 + (b - ccoef) * (b + ccoef)
      call FCAT_eels_all(299)
  if (dabs(a * c) > 1.0d-03 * b2) then
      call FCAT_eels_all(300)
    x = (b - dsqrt(b2 - a * c)) / a
  else
      call FCAT_eels_all(301)
      call FCAT_eels_all(302)
    x = a * c / b2
      call FCAT_eels_all(303)
    x = 0.5d0 * c * (1.d0 + 0.25d0 * x * (1.d0 + 0.5d0 * x * (1.d0 + 0.625d0 * x))) / b
      call FCAT_eels_all(304)
  endif
      call FCAT_eels_all(305)
  arg = x / u
      call FCAT_eels_all(306)
  if (dabs(arg) > 1.0d0) then
      call FCAT_eels_all(307)
    arg = dsign(1.0d0, arg)
      call FCAT_eels_all(308)
  endif
      call FCAT_eels_all(309)
  phi = dacos(arg)
      call FCAT_eels_all(310)
  fint2 = phint(phi, tanpsi, u)
      call FCAT_eels_all(311)
  if (rational) then
      call FCAT_eels_all(312)
    return
      call FCAT_eels_all(313)
  endif
      call FCAT_eels_all(314)
  if (user) then
      call FCAT_eels_all(315)
    fint2 = fint2 * usurlo(ru * u, wn)
  else
      call FCAT_eels_all(316)
      call FCAT_eels_all(317)
    fint2 = fint2 * surlos(ru * u, eps, thick, layers, nper)
      call FCAT_eels_all(318)
    if (dlimf > 0.0d0) then
      call FCAT_eels_all(319)
      t = ru * u * dlimf
      call FCAT_eels_all(320)
      fint2 = fint2 * (1.d0 + t * dlog(t / (t + 0.26d0)))**2 / (1.d0 + 1.40d0 * t)
      call FCAT_eels_all(321)
    endif
      call FCAT_eels_all(322)
  endif
      call FCAT_eels_all(323)
  return
end function fint2
double precision function fint3(u, eps, thick, layers, nper, eps_size)

! ******************************************************************
! *                                                                *
! * integration over the azimutal angle from phi1 > 0 to phi2 < pi *
! *                                                                *
! ******************************************************************

  implicit none

  double precision, intent(in) :: u
  integer, intent(in) :: layers, nper, eps_size
  double complex, intent(in) :: eps(eps_size)
  double precision, intent(in) :: thick(eps_size)

  logical :: rational, user
  double precision :: a, arg, b, ccoef, cospsi, dlimf, elleps, phi1, phi2
  double precision :: phint, sinpsi, rac, ru, um, t, tanpsi, wn

  interface
    double precision function usurlo(dq, wn)
      double precision, intent(in) :: dq
      double precision, intent(in) :: wn
    end function usurlo
    double precision function surlos(dk, eps, thick, layers, nper)
      double precision, intent(in) :: dk
      double complex, intent(in) :: eps(:)
      double precision, intent(in) :: thick(:)
      integer, intent(in) :: layers, nper
    end function surlos
  end interface  
  
  common / param / a, b, ccoef, elleps, cospsi, sinpsi, tanpsi,  &
      ru, um, dlimf, wn, user, rational

!  write (*,*) 'fint3:'
!  write (*,*) 'thick: ', size(thick)
!  write (*,*) 'eps: ', size(eps)

      call FCAT_eels_all(324)
  if (u == 0.0d0) then
      call FCAT_eels_all(325)
    fint3 = 0.0d0
      call FCAT_eels_all(326)
    return
      call FCAT_eels_all(327)
  endif
      call FCAT_eels_all(328)
  rac = dsign(1.0d0, a) * cospsi * dsqrt((1.0d0 - elleps) * a * (um - u) * (um + u))
      call FCAT_eels_all(329)
  arg = (b - rac) / (u * a)
      call FCAT_eels_all(330)
  if (dabs(arg) > 1.0d0) arg = dsign(1.0d0, arg)
      call FCAT_eels_all(331)
  phi2 = dacos(arg)
      call FCAT_eels_all(332)
  fint3 = phint(phi2, tanpsi, u)
      call FCAT_eels_all(333)
  arg = (b + rac) / (u * a)
      call FCAT_eels_all(334)
  if (dabs(arg) > 1.0d0) arg = dsign(1.0d0, arg)
      call FCAT_eels_all(335)
  phi1 = dacos(arg)
      call FCAT_eels_all(336)
  fint3 = fint3 - phint(phi1, tanpsi, u)
      call FCAT_eels_all(337)
  if (rational) return
      call FCAT_eels_all(338)
  if (user) then
      call FCAT_eels_all(339)
    fint3 = fint3 * usurlo(ru * u, wn)
  else
      call FCAT_eels_all(340)
      call FCAT_eels_all(341)
    fint3 = fint3 * surlos(ru * u, eps, thick, layers, nper)
      call FCAT_eels_all(342)
    if (dlimf > 0.0d0) then
      call FCAT_eels_all(343)
      t = ru * u * dlimf
      call FCAT_eels_all(344)
      fint3 = fint3 * (1.d0 + t * dlog(t / (t + 0.26d0)))**2 / (1.d0 + 1.40d0 * t)
      call FCAT_eels_all(345)
    endif
      call FCAT_eels_all(346)
  endif
      call FCAT_eels_all(347)
  return
end function fint3
double precision function fun(phi)

! ******************************************************************
! *                                                                *
! * integrand of the expression of the 1st order term in the       *
! * expansion of the eels integral for a homogeneous target.       *
! *                                                                *
! ******************************************************************

  implicit none

  double precision, intent(in) :: phi

  logical :: user, rational
  double precision :: acoef, bcoef, ccoef, cospsi, dlimf, elleps, ru
  double precision :: sinphi, sinpsi, tanpsi, um, wn

  common / param / acoef, bcoef, ccoef, elleps, cospsi, sinpsi, tanpsi,  &
                   ru, um, dlimf, wn, user, rational

      call FCAT_eels_all(348)
  sinphi = dsin(phi)
      call FCAT_eels_all(349)
  fun = dsqrt((1.0d0 - elleps + elleps * sinphi**2) *   &
              (1.0d0 - sinpsi * sinphi) *               &
              (1.0d0 + sinpsi * sinphi))
      call FCAT_eels_all(350)
  return
end function fun
subroutine quanc8(fun, a, b, abserr, relerr, result, errest, nofun, flag, eps, thick, layers, nper)

! estimate the integral of fun(x) from a to b
! to a user provided tolerance.
! an automatic adaptive routine based on
! the 8-panel newton-cotes rule (g. forsythe et al, 1977, p. 92)
!
! input ..
!
! fun     the name of the integrand function subprogram fun(x).
! a       the lower limit of integration.
! b       the upper limit of integration.(b may be less than a.)
! relerr  a relative error tolerance. (should be non-negative)
! abserr  an absolute error tolerance. (should be non-negative)
!
! output ..
!
! result  an approximation to the integral hopefully satisfying the
!         least stringent of the two error tolerances.
! errest  an estimate of the magnitude of the actual error.
! nofun   the number of function values used in calculation of result.
! flag    a reliability indicator.  if flag is zero, then result
!         probably satisfies the error tolerance.  if flag is
!         xxx.yyy , then  xxx = the number of intervals which have
!         not converged and 0.yyy = the fraction of the interval
!         left to do when the limit on  nofun  was approached.

  implicit none

  double precision :: fun
  double precision, intent(in) :: a
  double precision, intent(in) :: b
  double precision, intent(in out) :: abserr
  double precision, intent(in) :: relerr
  double precision, intent(out) :: result
  double precision, intent(out) :: errest
  integer, intent(out) :: nofun
  double precision, intent(out) :: flag

  external fun

  double precision, intent(in) :: thick(:)
  double complex, intent(in) :: eps(:)
  integer, intent(in) :: layers, nper

  double precision :: w0, w1, w2, w3, w4, area, x0, f0, stone, step, cor11, temp
  double precision :: qprev, qnow, qdiff, qleft, esterr, tolerr
  double precision :: qright(31), f(16), x(16), fsave(8, 30), xsave(8, 30)
  double precision :: dabs, dmax1

  integer :: levmin, levmax, levout, nomax, nofin, lev, nim, i, j

! ***   stage 1 ***   general initialization
! set constants.

!  write (*,*) 'quanc8:'
!  write (*,*) 'thick: ', size(thick)
!  write (*,*) 'eps: ', size(eps)

      call FCAT_eels_all(351)
  levmin = 1
      call FCAT_eels_all(352)
  levmax = 30
      call FCAT_eels_all(353)
  levout = 6
      call FCAT_eels_all(354)
  nomax = 5000
      call FCAT_eels_all(355)
  nofin = nomax - 8 * (levmax - levout + 2**(levout + 1))

! trouble when nofun reaches nofin

      call FCAT_eels_all(356)
  w0 =   3956.0d0 / 14175.0d0
      call FCAT_eels_all(357)
  w1 =  23552.0d0 / 14175.0d0
      call FCAT_eels_all(358)
  w2 =  -3712.0d0 / 14175.0d0
      call FCAT_eels_all(359)
  w3 =  41984.0d0 / 14175.0d0
      call FCAT_eels_all(360)
  w4 = -18160.0d0 / 14175.0d0

! initialize running sums to zero.

      call FCAT_eels_all(361)
  flag   = 0.0d0
      call FCAT_eels_all(362)
  result = 0.0d0
      call FCAT_eels_all(363)
  cor11  = 0.0d0
      call FCAT_eels_all(364)
  errest = 0.0d0
      call FCAT_eels_all(365)
  area   = 0.0d0
      call FCAT_eels_all(366)
  nofun = 0
      call FCAT_eels_all(367)
  if (a == b) return

! ***   stage 2 ***   initialization for first interval

      call FCAT_eels_all(368)
  lev = 0
      call FCAT_eels_all(369)
  nim = 1
      call FCAT_eels_all(370)
  x0 = a
      call FCAT_eels_all(371)
  x(16) = b
      call FCAT_eels_all(372)
  qprev  = 0.0d0
      call FCAT_eels_all(373)
  f0 = fun(x0, eps, thick, layers, nper, size(eps))
      call FCAT_eels_all(374)
  stone = (b - a) / 16
      call FCAT_eels_all(375)
  x(8)  =  (x0    + x(16)) / 2
      call FCAT_eels_all(376)
  x(4)  =  (x0    + x(8))  / 2
      call FCAT_eels_all(377)
  x(12) =  (x(8)  + x(16)) / 2
      call FCAT_eels_all(378)
  x(2)  =  (x0    + x(4))  / 2
      call FCAT_eels_all(379)
  x(6)  =  (x(4)  + x(8))  / 2
      call FCAT_eels_all(380)
  x(10) =  (x(8)  + x(12)) / 2
      call FCAT_eels_all(381)
  x(14) =  (x(12) + x(16)) / 2
      call FCAT_eels_all(382)
  do j = 2, 16, 2
      call FCAT_eels_all(383)
    f(j) = fun(x(j), eps, thick, layers, nper, size(eps))
      call FCAT_eels_all(384)
  enddo
      call FCAT_eels_all(385)
  nofun = 9

      call FCAT_eels_all(386)
  do

! ***   stage 3 ***   central calculation
! requires qprev, x0, x2, x4, ..., x16, f0, f2, f4, ..., f16.
! calculates x1, x3, ...x15, f1, f3, ...f15, qleft, qright, qnow, qdiff, area.

      call FCAT_eels_all(387)
    x(1) = (x0 + x(2)) / 2
      call FCAT_eels_all(388)
    f(1) = fun(x(1), eps, thick, layers, nper, size(eps))
      call FCAT_eels_all(389)
    do j = 3, 15, 2
      call FCAT_eels_all(390)
      x(j) = (x(j - 1) + x(j + 1)) / 2
      call FCAT_eels_all(391)
      f(j) = fun(x(j), eps, thick, layers, nper, size(eps))
      call FCAT_eels_all(392)
    enddo
      call FCAT_eels_all(393)
    nofun = nofun + 8
      call FCAT_eels_all(394)
    step = (x(16) - x0) / 16.0d0
      call FCAT_eels_all(395)
    qleft  =  (w0 * (f0 + f(8))  + w1 * (f(1) + f(7)) + w2 * (f(2) + f(6))  &
        + w3 * (f(3) + f(5))  +  w4 * f(4)) * step
      call FCAT_eels_all(396)
    qright(lev + 1) = (w0 * (f(8) + f(16)) + w1 * (f(9) + f(15)) + w2 * (f(10) + f(14))  &
        + w3 * (f(11) + f(13)) + w4 * f(12)) * step
      call FCAT_eels_all(397)
    qnow = qleft + qright(lev + 1)
      call FCAT_eels_all(398)
    qdiff = qnow - qprev
      call FCAT_eels_all(399)
    area = area + qdiff

! ***   stage 4 *** interval convergence test

      call FCAT_eels_all(400)
    esterr = dabs(qdiff) / 1023
      call FCAT_eels_all(401)
    tolerr = dmax1(abserr, relerr * dabs(area)) * (step / stone)

      call FCAT_eels_all(402)
    if (lev >= levmin) then
      call FCAT_eels_all(403)
      if (lev >= levmax) then
! current level is levmax.
      call FCAT_eels_all(404)
        flag = flag + 1.0d0
      else
      call FCAT_eels_all(405)
      call FCAT_eels_all(406)
        if (nofun > nofin) then
! ***   stage 6   ***   trouble section
! number of function values is about to exceed limit.
      call FCAT_eels_all(407)
          nofin = 2 * nofin
      call FCAT_eels_all(408)
          levmax = levout
      call FCAT_eels_all(409)
          flag = flag + (b - x0) / (b - a)
        else
      call FCAT_eels_all(410)
      call FCAT_eels_all(411)
          if (esterr > tolerr) then
! ***   stage 5   ***   no convergence
! locate next interval.
      call FCAT_eels_all(412)
            nim = 2 * nim
      call FCAT_eels_all(413)
            lev = lev + 1
! store right hand elements for future use.
      call FCAT_eels_all(414)
            do i = 1, 8
      call FCAT_eels_all(415)
              fsave(i, lev) = f(i + 8)
      call FCAT_eels_all(416)
              xsave(i, lev) = x(i + 8)
      call FCAT_eels_all(417)
            enddo
! assemble left hand elements for immediate use.
      call FCAT_eels_all(418)
            qprev = qleft
      call FCAT_eels_all(419)
            do i = 1, 8
      call FCAT_eels_all(420)
              f(18 - 2 * i) = f(9 - i)
      call FCAT_eels_all(421)
              x(18 - 2 * i) = x(9 - i)
      call FCAT_eels_all(422)
            enddo
      call FCAT_eels_all(423)
            cycle
      call FCAT_eels_all(424)
          endif
      call FCAT_eels_all(425)
        endif
      call FCAT_eels_all(426)
      endif

! ***   stage 7   ***   interval converged
! add contributions into running sums.
      call FCAT_eels_all(427)
      result = result + qnow
      call FCAT_eels_all(428)
      errest = errest + esterr
      call FCAT_eels_all(429)
      cor11  = cor11  + qdiff / 1023
! locate next interval.
      call FCAT_eels_all(430)
      do while (nim /= 2 * (nim / 2))
      call FCAT_eels_all(431)
        nim = nim / 2
      call FCAT_eels_all(432)
        lev = lev - 1
      call FCAT_eels_all(433)
      enddo
      call FCAT_eels_all(434)
      nim = nim + 1

      call FCAT_eels_all(435)
      if (lev <= 0) exit

! assemble elements required for the next interval.
      call FCAT_eels_all(436)
      qprev = qright(lev)
      call FCAT_eels_all(437)
      x0 = x(16)
      call FCAT_eels_all(438)
      f0 = f(16)
      call FCAT_eels_all(439)
      do i = 1, 8
      call FCAT_eels_all(440)
        f(2*i) = fsave(i, lev)
      call FCAT_eels_all(441)
        x(2*i) = xsave(i, lev)
      call FCAT_eels_all(442)
      enddo
      call FCAT_eels_all(443)
      cycle
    else
! ***   stage 5   ***   no convergence
! locate next interval.
      call FCAT_eels_all(444)
      call FCAT_eels_all(445)
      nim = 2 * nim
      call FCAT_eels_all(446)
      lev = lev + 1
! store right hand elements for future use.
      call FCAT_eels_all(447)
      do i = 1, 8
      call FCAT_eels_all(448)
        fsave(i, lev) = f(i + 8)
      call FCAT_eels_all(449)
        xsave(i, lev) = x(i + 8)
      call FCAT_eels_all(450)
      enddo
! assemble left hand elements for immediate use.
      call FCAT_eels_all(451)
      qprev = qleft
      call FCAT_eels_all(452)
      do i = 1, 8
      call FCAT_eels_all(453)
        f(18 - 2 * i) = f(9 - i)
      call FCAT_eels_all(454)
        x(18 - 2 * i) = x(9 - i)
      call FCAT_eels_all(455)
      enddo
      call FCAT_eels_all(456)
    endif

      call FCAT_eels_all(457)
   enddo

 ! ***   stage 8   ***   finalize and return
      call FCAT_eels_all(458)
  result = result + cor11

! make sure errest not less than roundoff level.
      call FCAT_eels_all(459)
  if (errest /= 0.0d0) then
      call FCAT_eels_all(460)
    temp = dabs(result) + errest
      call FCAT_eels_all(461)
    do while (temp == dabs(result))
      call FCAT_eels_all(462)
      errest = 2 * errest
      call FCAT_eels_all(463)
      temp = dabs(result) + errest
      call FCAT_eels_all(464)
    enddo
      call FCAT_eels_all(465)
  endif
      call FCAT_eels_all(466)
  return
end subroutine quanc8
subroutine queels(x, f, aerr, rerr, facru, eps, thick, layers, nper)

! ******************************************************************
! *                                                                *
! * perform q-space integration for computing the eels spectrum of *
! * a isotropic target using polar coordinates.                    *
! *                                                                *
! * x is the dimensionless energy loss hbar*omega/(2*e0*phia)      *
! * aerr and rerr are the desired absolute and relative accuracies *
! * facru*x is the units of wavevectors omega/v_perpendicular      *
! * f is the q-integral multiplied by (2/pi)**2                    *
! *                                                                *
! ******************************************************************

  implicit none

  double precision, intent(in) :: x
  double precision, intent(out) :: f
  double precision, intent(in out) :: aerr
  double precision, intent(in out) :: rerr
  double precision, intent(in) :: facru
  double complex, intent(in) :: eps(:)
  double precision, intent(in) :: thick(:)
  integer, intent(in) :: layers, nper

  logical :: rational, user
  double precision :: acoef, bcoef, ccoef, cospsi, dlimf, elleps
  double precision :: error, flag, ru, sinpsi
  double precision :: u1, u2, um, ut, tanpsi, wn, y
  integer :: ie, nofu
  dimension error(3), flag(3)

  interface
    double precision function fint1(u, eps, thick, layers, nper, eps_size)
      double precision, intent(in) :: u
      integer, intent(in) :: layers, nper, eps_size
      double complex, intent(in) :: eps(eps_size)
      double precision, intent(in) :: thick(eps_size)
    end function fint1
    double precision function fint2(u, eps, thick, layers, nper, eps_size)
      double precision, intent(in) :: u
      integer, intent(in) :: layers, nper, eps_size
      double complex, intent(in) :: eps(eps_size)
      double precision, intent(in) :: thick(eps_size)
    end function fint2
    double precision function fint3(u, eps, thick, layers, nper, eps_size)
      double precision, intent(in) :: u
      integer, intent(in) :: layers, nper, eps_size
      double complex, intent(in) :: eps(eps_size)
      double precision, intent(in) :: thick(eps_size)
    end function fint3
    subroutine quanc8(fun, a, b, abserr, relerr, result, errest, nofun, flag, eps, thick, layers, nper)
      double precision :: fun
      double precision, intent(in) :: a
      double precision, intent(in) :: b
      double precision, intent(in out) :: abserr
      double precision, intent(in) :: relerr
      double precision, intent(out) :: result
      double precision, intent(out) :: errest
      integer, intent(out) :: nofun
      double precision, intent(out) :: flag

      external fun

      double precision, intent(in) :: thick(:)
      double complex, intent(in) :: eps(:)
      integer, intent(in) :: layers, nper
    end subroutine quanc8  
  end interface  

  common / param / acoef, bcoef, ccoef, elleps, cospsi, sinpsi, tanpsi,  &
      ru, um, dlimf, wn, user, rational

!  write (*,*) 'queels:'
!  write (*,*) 'eps: ', size(eps)
!  write (*,*) 'thick: ', size(thick)

      call FCAT_eels_all(467)
  f = 0.0d0
      call FCAT_eels_all(468)
  if (x <= 0.0d0) then
      call FCAT_eels_all(469)
    return
      call FCAT_eels_all(470)
  endif
      call FCAT_eels_all(471)
  ru = facru * x
      call FCAT_eels_all(472)
  ccoef = cospsi**2 / x
      call FCAT_eels_all(473)
  ut = ccoef - bcoef
      call FCAT_eels_all(474)
  u1 = dabs(ut)
      call FCAT_eels_all(475)
  u2 = ccoef + bcoef
      call FCAT_eels_all(476)
  if (ut > 0.0d0) then
      call FCAT_eels_all(477)
    call quanc8(fint1, 0.0d0, u1, aerr, rerr, y, error(1), nofu, flag(1), eps, thick, layers, nper)
      call FCAT_eels_all(478)
    f = y
  else
      call FCAT_eels_all(479)
      call FCAT_eels_all(480)
    flag(1) = 0.0d0
      call FCAT_eels_all(481)
  endif
      call FCAT_eels_all(482)
  if (u2 > u1) then
      call FCAT_eels_all(483)
    call quanc8(fint2, u1, u2, aerr, rerr, y, error(2), nofu, flag(2), eps, thick, layers, nper)
      call FCAT_eels_all(484)
    f = f + y
  else
      call FCAT_eels_all(485)
      call FCAT_eels_all(486)
    flag(2) = 0.0d0
      call FCAT_eels_all(487)
  endif
      call FCAT_eels_all(488)
  if (dabs(acoef) > x * (1.0d0 - elleps) * bcoef) then
      call FCAT_eels_all(489)
    um = dsqrt(ccoef / x / (1.0d0 - elleps) + bcoef**2 / acoef)
      call FCAT_eels_all(490)
    if (um > u2) then
      call FCAT_eels_all(491)
      call quanc8(fint3, u2, um, aerr, rerr, y, error(3), nofu, flag(3), eps, thick, layers, nper)
      call FCAT_eels_all(492)
      f = f + y
      call FCAT_eels_all(493)
    endif
      call FCAT_eels_all(494)
    if (um < u1) then
      call FCAT_eels_all(495)
      call quanc8(fint3, um, u1, aerr, rerr, y, error(3), nofu, flag(3), eps, thick, layers, nper)
      call FCAT_eels_all(496)
      f = f - y
      call FCAT_eels_all(497)
    endif
  else
      call FCAT_eels_all(498)
      call FCAT_eels_all(499)
    flag(3) = 0.0d0
      call FCAT_eels_all(500)
  endif
      call FCAT_eels_all(501)
  do ie = 1, 3
      call FCAT_eels_all(502)
    if (flag(ie) == 0.0d0) cycle
      call FCAT_eels_all(503)
    write(*,*) ' +++ flag(', ie, ') =', flag(ie), ', error =', error(ie), ' +++'
      call FCAT_eels_all(504)
    if (flag(ie) - aint(flag(ie)) > 0.5d-02) then
      call FCAT_eels_all(505)
      call FCAT_eels_all_rep()
      stop '*** execution aborted ***'
      call FCAT_eels_all(506)
    endif
      call FCAT_eels_all(507)
  enddo
      call FCAT_eels_all(508)
  f = (2 / 3.141592653589793238d0)**2 * f
      call FCAT_eels_all(509)
  return
end subroutine queels
subroutine seteps(neps, nos, osc, epsinf, wn, name, eps, layers, mode)

! ******************************************************************
! *                                                                *
! * set up long-wavelength dielectric functions of the layers for  *
! * the present frequency wn (in cm**-1)                           *
! *                                                                *
! ******************************************************************

  implicit none
  integer, intent(in) :: neps
  integer, intent(in) :: nos(:)
  double precision, intent(in) :: osc(:, :)
  double precision, intent(in) :: epsinf(:)
  double precision, intent(in) :: wn
  character (len=10), intent(in) :: name(:)
  character (len=10), intent(in) :: mode

  double complex, intent(in out) :: eps(:)
  integer, intent(in) :: layers

  double precision :: argmin, argmax, epsmac, x
  double complex :: deno, nomi
  integer :: j, k, l, m

  common / mulayr / argmin, argmax, epsmac

!  write (*,*) 'seteps:'
!  write (*,*) 'nos: ', size(nos)
!  write (*,*) 'osc: ', size(osc)
!  write (*,*) 'epsinf: ', size(epsinf)
!  write (*,*) 'name: ', size(name)
!  write (*,*) 'eps: ', size(eps)
!  write (*,*) 'thick: ', size(thick)

      call FCAT_eels_all(510)
  j = 0
      call FCAT_eels_all(511)
  do l = 1, neps
      call FCAT_eels_all(512)
    m = nos(l)/2
      call FCAT_eels_all(513)
    nomi = dcmplx(1.0d0, 0.0d0)
      call FCAT_eels_all(514)
    deno = dcmplx(1.0d0, 0.0d0)
      call FCAT_eels_all(515)
    if (mode == 'kurosawa') then
      call FCAT_eels_all(516)
      do k = 1, m
      call FCAT_eels_all(517)
        j = j + 1
! since osc(1,*) and wn are real, the following should be equivalent
! Check required.
! Furthermore the first term can be rewritten as
! (osc(1, j + m) - wn) * (osc(1, j + m) + wn)
!        nomi = nomi * cmplx(osc(1, j + m)**2 - wn**2, - wn * osc(3, j + m))
      call FCAT_eels_all(518)
        nomi = nomi * (osc(1, j + m)**2 - wn**2 - dcmplx(0.0d0, wn * osc(3, j + m)))
      call FCAT_eels_all(519)
        deno = deno * (osc(1, j    )**2 - wn**2 - dcmplx(0.0d0, wn * osc(3, j    )))
      call FCAT_eels_all(520)
      enddo
      call FCAT_eels_all(521)
      eps(l) = epsinf(l) * nomi / deno
    elseif (name(l) == 'metal') then
      call FCAT_eels_all(522)
      call FCAT_eels_all(523)
      j = j + 1
! suggestion for replacement
!      eps(l) = -osc(1, j)**2 / cmplx(wn**2, wn * osc(3, j))
      call FCAT_eels_all(524)
      eps(l) = -osc(1, j)**2 / ( wn**2 + dcmplx(0.0d0, wn * osc(3, j)) )
    else
      call FCAT_eels_all(525)
      call FCAT_eels_all(526)
      eps(l) = epsinf(l)
! The original version had this additional loop. It seems, it has been removed
! because all cases of nos(l) > 1 are treated in case 1 above
      call FCAT_eels_all(527)
      do k = 1, nos(l)
      call FCAT_eels_all(528)
      j = j + 1
      call FCAT_eels_all(529)
      x = wn / osc(1, j)
      call FCAT_eels_all(530)
      deno = x * dcmplx(x, osc(3, j))
      call FCAT_eels_all(531)
      if (osc(2, j) >= 0.0d0) then
      call FCAT_eels_all(532)
        deno = 1.0d0 - deno
      call FCAT_eels_all(533)
      endif
      call FCAT_eels_all(534)
      if (cdabs(deno) == 0.0d0) then
      call FCAT_eels_all(535)
        deno = epsmac
      call FCAT_eels_all(536)
      endif
      call FCAT_eels_all(537)
      eps(l) = eps(l) + osc(2, j) / deno
      call FCAT_eels_all(538)
      enddo
      call FCAT_eels_all(539)
    endif
      call FCAT_eels_all(540)
  enddo
      call FCAT_eels_all(541)
  if (neps == layers + 1) then
! the substrate is a anisotropic uniaxial material
      call FCAT_eels_all(542)
    eps(layers) = cdsqrt(eps(layers) * eps(layers + 1))
      call FCAT_eels_all(543)
    if (dimag(eps(layers)) < 0.0d0) then
      call FCAT_eels_all(544)
      eps(layers) = -eps(layers)
      call FCAT_eels_all(545)
    endif
      call FCAT_eels_all(546)
  endif
      call FCAT_eels_all(547)
  return
end subroutine seteps
subroutine doeels (e0, theta, phia, phib, wmin, wmax, dw, comment, comment_size,   &
              layers, neps, nper, name, name_size, thick, epsinf, nos, osc, osc_size,&
              contrl, mode, wn_array, f_array, wn_array_size)

! ******************************************************************
! *                                                                *
! * compute the classical eels spectrum of an arbitrary plane-     *
! * statified medium made from isotropic materials in specular     *
! * geometry using the dielectric theory of eels.                  *
! *                                                                *
! ******************************************************************

  implicit none

  integer, parameter :: nt = 5

  double precision, intent(in) :: e0, theta, phia, phib, wmin, wmax, dw
  character (len = 72) :: comment(comment_size)
  character (len = 10) :: name(name_size)
  double precision, intent(in) :: thick(name_size), epsinf(name_size), osc(3, osc_size)
  character (len = 10) :: contrl, mode
  integer, intent(in) :: comment_size, name_size, osc_size, wn_array_size
  integer, intent(in out) :: layers, nper, nos(name_size)
  double precision, intent(in out) :: wn_array(wn_array_size), f_array(wn_array_size)

  logical :: rational, user, debug
  integer :: i, iw, neps, nofu, nout, nw, lmax
  double precision :: a, acoef, aerr, alpha, argmin, argmax, b, bcoef, beta,  &
      c1, c2, ccoef, cospsi, dlimf, dx, elleps, ener, epsmac, facru, f, f0,   &
      f1, fpic, fun, pi, prefac, psia, psii, qrat, rerr, ru, sinpsi, t,       &
      tanpsi, table, um, widt, wn, wpic, x, xmin, xmax, z, z1, z2
  double complex, allocatable :: eps(:)
  dimension table(nt)

  external fun, qrat

  interface
    subroutine queels(x, f, aerr, rerr, facru, eps, thick, layers, nper)
      double precision, intent(in) :: x
      double precision, intent(out) :: f
      double precision, intent(in out) :: aerr
      double precision, intent(in out) :: rerr
      double precision, intent(in) :: facru
      double complex, intent(in) :: eps(:)
      double precision, intent(in) :: thick(:)
      integer, intent(in) :: layers, nper
    end subroutine queels
    subroutine seteps(neps, nos, osc, epsinf, wn, name, eps, layers, mode)
      integer, intent(in) :: neps
      integer, intent(in) :: nos(:)
      double precision, intent(in) :: osc(:, :)
      double precision, intent(in) :: epsinf(:)
      double precision, intent(in) :: wn
      character (len=10), intent(in) :: name(:)
      character (len=10), intent(in) :: mode
      double complex, intent(in out) :: eps(:)
      integer, intent(in) :: layers
    end subroutine seteps
    subroutine quanc8(fun, a, b, abserr, relerr, result, errest, nofun, flag, eps, thick, layers, nper)
      double precision :: fun
      double precision, intent(in) :: a
      double precision, intent(in) :: b
      double precision, intent(in out) :: abserr
      double precision, intent(in) :: relerr
      double precision, intent(out) :: result
      double precision, intent(out) :: errest
      integer, intent(out) :: nofun
      double precision, intent(out) :: flag

      external fun

      double precision, intent(in) :: thick(:)
      double complex, intent(in) :: eps(:)
      integer, intent(in) :: layers, nper
    end subroutine quanc8  
  end interface

  common / param / acoef, bcoef, ccoef, elleps, cospsi, sinpsi, tanpsi,  &
                   ru, um, dlimf, wn, user, rational
  common / mulayr / argmin, argmax, epsmac

  data aerr / 0.0d0 /, rerr / 1.0d-06 /, f / 0.0d0 /, f1 / 0.0d0 /

      call FCAT_eels_all(548)
  debug = .false.
      call FCAT_eels_all(549)
  if (debug) then
      call FCAT_eels_all(550)
    write (*,*) 'doeels:'
      call FCAT_eels_all(551)
    write (*,*) 'comment: ',  size(comment)
      call FCAT_eels_all(552)
    write (*,*) 'name: ',     size(name)
      call FCAT_eels_all(553)
    write (*,*) 'thick: ',    size(thick)
      call FCAT_eels_all(554)
    write (*,*) 'epsinf: ',   size(epsinf)
      call FCAT_eels_all(555)
    write (*,*) 'osc: ',      size(osc), size(osc, 1), size(osc, 2)
      call FCAT_eels_all(556)
    write (*,*) 'nos: ',      size(nos)
      call FCAT_eels_all(557)
    write (*,*) 'wn_array: ', size(wn_array)
      call FCAT_eels_all(558)
    write (*,*) 'f_array: ',  size(f_array)
      call FCAT_eels_all(559)
  endif

! *** machine-dependent constants
! *** epsmac + 1.0 = epsmac , cosh(argmin) = 1.0 , tanh(argmax) = 1.0

      call FCAT_eels_all(560)
  pi = 4 * datan(1.0d0)
      call FCAT_eels_all(561)
  epsmac = 1.0d0
      call FCAT_eels_all(562)
  do while (1.0d0 + epsmac > 1.0d0)
      call FCAT_eels_all(563)
    epsmac = epsmac / 2
      call FCAT_eels_all(564)
  enddo
      call FCAT_eels_all(565)
  argmin = dsqrt(2 * epsmac)
      call FCAT_eels_all(566)
  argmax = 0.5d0 * dlog(2 / epsmac)

      call FCAT_eels_all(567)
  dlimf = 0.0d0
      call FCAT_eels_all(568)
  rational = .false.

! *** read target specifications

      call FCAT_eels_all(569)
  user = layers == 0
      call FCAT_eels_all(570)
  if (user) then

      call FCAT_eels_all(571)
    if (layers == 1) rational = .true.
      call FCAT_eels_all(572)
    if (contrl == 'image') then
! *** image-charge screening factor
      call FCAT_eels_all(573)
      if (layers == 1 .and. neps == 2) then
      call FCAT_eels_all(574)
        dlimf = dsqrt(epsinf(1) * epsinf(2))
      else
      call FCAT_eels_all(575)
      call FCAT_eels_all(576)
        dlimf = epsinf(1)
      call FCAT_eels_all(577)
      endif
      call FCAT_eels_all(578)
      dlimf = (dlimf - 1.0d0) / (dlimf + 1.0d0)
      call FCAT_eels_all(579)
    endif
      call FCAT_eels_all(580)
  endif

! *** initialize constants

      call FCAT_eels_all(581)
  lmax = size(thick)
      call FCAT_eels_all(582)
  nw = size(wn_array)
      call FCAT_eels_all(583)
  if (debug) write (*,*) 'lmax: ', lmax
      call FCAT_eels_all(584)
  allocate(eps(lmax))
      call FCAT_eels_all(585)
  if (debug) write (*,*) 'eps: ', size(eps)
      call FCAT_eels_all(586)
  nout = 1 + nw / 20
      call FCAT_eels_all(587)
  ener = 8065 * e0
      call FCAT_eels_all(588)
  psia = phia / 180 * pi
      call FCAT_eels_all(589)
  psii = theta / 180 * pi
      call FCAT_eels_all(590)
  cospsi = dcos(psii)
      call FCAT_eels_all(591)
  sinpsi = dsin(psii)
      call FCAT_eels_all(592)
  tanpsi = dtan(psii)
      call FCAT_eels_all(593)
  prefac = dsqrt(255500 / e0)/(137 * cospsi)
      call FCAT_eels_all(594)
  facru = psia / cospsi * dsqrt(0.2624664d0 * e0)
      call FCAT_eels_all(595)
  elleps = (1.0d0 - phia / phib) * (1.0d0 + phia / phib)
      call FCAT_eels_all(596)
  acoef = sinpsi**2 + elleps * cospsi**2
      call FCAT_eels_all(597)
  bcoef = sinpsi * cospsi
      call FCAT_eels_all(598)
  if (dlimf > 0.0d0) then
      call FCAT_eels_all(599)
    rational = .false.
      call FCAT_eels_all(600)
    if (debug) write(*,*) ' = > electron attracted by an image charge = ', dlimf
! *** dlimf : half the length unit imposed by the image force
      call FCAT_eels_all(601)
    dlimf = 1.80d0 * dlimf/(e0 * cospsi**2)
      call FCAT_eels_all(602)
  endif
      call FCAT_eels_all(603)
  if (debug) write (*,*) 'rational: ', rational
      call FCAT_eels_all(604)
  if (rational) then

! *** set up coefficients for the rational approximation to the integral

      call FCAT_eels_all(605)
    if (debug) write(*,*) ' = > set up a rational approximation to the integral'
      call FCAT_eels_all(606)
    call quanc8(fun, 0.0d0, pi / 2, aerr, rerr, alpha, c1, nofu, c2, eps, thick, layers, nper)
      call FCAT_eels_all(607)
    alpha  = (2 / pi)**2 * alpha
      call FCAT_eels_all(608)
    c1 = 2 / pi / dsqrt(1.0d0 - elleps) * sinpsi * alpha**2
      call FCAT_eels_all(609)
    if (c1 > 0.99d0) then
      call FCAT_eels_all(610)
      if (debug) write(*,*) ' ===> cannot do it'
      call FCAT_eels_all(611)
      rational = .false.
    else
      call FCAT_eels_all(612)
      call FCAT_eels_all(613)
      c2 = 3 * alpha**2 / (1.0d0 - c1)
      call FCAT_eels_all(614)
      c1 = c1 * c2
      call FCAT_eels_all(615)
      xmin = wmin / (2 * ener * psia)
      call FCAT_eels_all(616)
      xmax = wmax / (2 * ener * psia)
      call FCAT_eels_all(617)
      if (xmin <= 0.0d0) xmin = 0.0d0
      call FCAT_eels_all(618)
      dx = dmax1(0.02d0, (xmax - xmin) / nt)
      call FCAT_eels_all(619)
      z1 = 0.0d0
      call FCAT_eels_all(620)
      z2 = 0.0d0
      call FCAT_eels_all(621)
      do i = 1, nt
      call FCAT_eels_all(622)
        x = xmin + i * dx
      call FCAT_eels_all(623)
        call queels(x, f, aerr, rerr, facru, eps, thick, layers, nper)
      call FCAT_eels_all(624)
        table(i) = f
      call FCAT_eels_all(625)
        f = f * (1.0d0 + alpha * x)**2
      call FCAT_eels_all(626)
        if (dabs(c2 * f - c1) < c2 * rerr) cycle
      call FCAT_eels_all(627)
        z = (1.0d0 - f) / (c2 * f - c1)
      call FCAT_eels_all(628)
        if (z <= 0.0d0) cycle
      call FCAT_eels_all(629)
        z1 = z1 + x * z * (x**2 - z)
      call FCAT_eels_all(630)
        z2 = z2 + (x * z)**2
      call FCAT_eels_all(631)
      enddo
      call FCAT_eels_all(632)
      if (z2 == 0.0d0) then
      call FCAT_eels_all(633)
        if (debug) write(*,*) ' ===> cannot do it'
      call FCAT_eels_all(634)
        rational = .false.
      else
      call FCAT_eels_all(635)
      call FCAT_eels_all(636)
        beta = z1 / z2
      call FCAT_eels_all(637)
        z = 0.0d0
      call FCAT_eels_all(638)
        do i = 1, nt
      call FCAT_eels_all(639)
          x = xmin + i * dx
      call FCAT_eels_all(640)
          z = z + (table(i) - qrat(x, alpha, beta, c1, c2))**2
      call FCAT_eels_all(641)
        enddo
      call FCAT_eels_all(642)
        z = dsqrt(z) / nt
      call FCAT_eels_all(643)
        if (z > 5.0d-03) then
      call FCAT_eels_all(644)
          if (debug) write(*,*) ' ===> cannot do it'
      call FCAT_eels_all(645)
          rational = .false.
        else
      call FCAT_eels_all(646)
      call FCAT_eels_all(647)
          if (debug) write(*, 100) alpha, c1, c2, beta, z
      call FCAT_eels_all(648)
        endif ! z > 5.0d-03
      call FCAT_eels_all(649)
      endif ! z2 == 0.0d0
      call FCAT_eels_all(650)
    endif ! c1 > 0.99d0
      call FCAT_eels_all(651)
  endif ! rational

! *** loop over the energy losses

      call FCAT_eels_all(652)
  if (debug) write(*, 110)
      call FCAT_eels_all(653)
  do iw = 1, nw
      call FCAT_eels_all(654)
    f0 = f1
      call FCAT_eels_all(655)
    f1 = f
      call FCAT_eels_all(656)
    f = 0.0d0
      call FCAT_eels_all(657)
    wn = wmin + (iw - 1) * dw
!    if (debug) write (*,*) 'wn: ', wn
      call FCAT_eels_all(658)
    if (wn >= 0.0d0) then
      call FCAT_eels_all(659)
      if (wn /= 0.0d0) then
      call FCAT_eels_all(660)
        if (.not. user) call seteps(neps, nos, osc, epsinf, wn, name, eps, layers, mode)

      call FCAT_eels_all(661)
        x = wn / (2 * ener * psia)
      call FCAT_eels_all(662)
        if (rational) then
      call FCAT_eels_all(663)
          f = qrat(x, alpha, beta, c1, c2) * dimag(-2 / (1.0d0 + eps(1)))
        else
      call FCAT_eels_all(664)
      call FCAT_eels_all(665)
          call queels(x, f, aerr, rerr, facru, eps, thick, layers, nper)
      call FCAT_eels_all(666)
        endif
      call FCAT_eels_all(667)
        f = prefac * f / wn
      call FCAT_eels_all(668)
      endif ! wn /= 0.0d0

      call FCAT_eels_all(669)
      wn_array(iw) = wn
      call FCAT_eels_all(670)
      f_array(iw)  = f

! *** localize a peak using a parabolic interpolation

      call FCAT_eels_all(671)
      if ((iw >= 3) .and. (f1 - f0 > aerr) .and. (f1 - f > aerr)) then
      call FCAT_eels_all(672)
        a = (f1 - f0) + (f1 - f)
      call FCAT_eels_all(673)
        if (a > 4 * rerr * f1) then
      call FCAT_eels_all(674)
          b = 0.5d0 * (f1 - f0 + 3 * (f1 - f))
      call FCAT_eels_all(675)
          t = b / a
      call FCAT_eels_all(676)
          wpic = wn - t * dw
      call FCAT_eels_all(677)
          fpic = f + 0.5d0 * b * t
      call FCAT_eels_all(678)
          widt = dsqrt(8 * fpic / a) * dw
      call FCAT_eels_all(679)
          if (debug) write(*, 120) wpic, fpic, widt
      call FCAT_eels_all(680)
        endif ! a > 4 * rerr * f1
      call FCAT_eels_all(681)
      endif ! iw >= 3 ...
      call FCAT_eels_all(682)
    endif ! wn >= 0.0d0
      call FCAT_eels_all(683)
    if (mod(iw, nout) == 0) then
      call FCAT_eels_all(684)
      if (debug) write(*, 130) 100 * iw / nw, wn, f
      call FCAT_eels_all(685)
    endif
      call FCAT_eels_all(686)
  enddo
      call FCAT_eels_all(687)
  return
      call FCAT_eels_all(688)
100 format(5x, 'alpha = ', f9.4, 4x, 'c1 = ', f9.4, 4x, 'c2 = ', f9.4, 4x,  &
      'beta = ', f9.4/5x, 'accuracy = ', e9.2)
      call FCAT_eels_all(689)
110 format(//' run (%)  wn (cm**-1)  pcl(wn) (cm) |',  &
      ' peak location  amplitude    width')
      call FCAT_eels_all(690)
120 format(40x, f10.2, d12.4, f10.2)
      call FCAT_eels_all(691)
130 format(2x, f5.1, 3x, f11.3, d14.5)
end subroutine doeels
subroutine change_working_dir()

! This routine gets the first argument of the commandline and takes it
! as the path to change the working directory
! used intrinsic routines:
! iarg returns the number of commandline arguments without the program cname.
! chdir changes the directory and returns 0 on success.
! trim removes blanks from strings.

  character (len = 256) :: argument
  integer :: status

      call FCAT_eels_all(692)
  if (iargc() == 1) then
      call FCAT_eels_all(693)
    call getarg(1, argument)
      call FCAT_eels_all(694)
    status = chdir(trim(argument))
      call FCAT_eels_all(695)
    if (status /= 0) then
      call FCAT_eels_all(696)
      write (*,*) '*** change directory failed ***'
      call FCAT_eels_all(697)
      write (*,*) 'directory tried: ', trim(argument)
      call FCAT_eels_all(698)
      write (*,*) 'error code (see: man chdir): ', status
      call FCAT_eels_all(699)
      write (*,*) 'continuing in the start directory'
      call FCAT_eels_all(700)
    end if
      call FCAT_eels_all(701)
  end if

      call FCAT_eels_all(702)
  return
end subroutine change_working_dir
       module FCAT_eels_all_mod
        double precision,dimension (703):: &
     &   FCAT_eels_all_counter = 0
        integer :: FCAT_eels_all_nline = 702
       end module FCAT_eels_all_mod
       subroutine FCAT_eels_all(n)
         use FCAT_eels_all_mod
         integer :: n
         FCAT_eels_all_counter(n) = &
     &    FCAT_eels_all_counter(n)  + 1
         if (FCAT_eels_all_counter(n) == 1) then
           write(*,"(a,i10)") "FCAT_eels_all_",n
         endif
       end subroutine FCAT_eels_all
       subroutine FCAT_eels_all_rep()
          use FCAT_eels_all_mod
          integer :: i
          do i = 1, FCAT_eels_all_nline
             write(*,"(a,i10,i10)") &
     &      "FCAT_eels_all_count",i, &
     &      int(FCAT_eels_all_counter(i)+0.1)
          end do
       end subroutine FCAT_eels_all_rep
