      SUBROUTINE QUEELS(X,F,AERR,RERR,FACRU)                            
C    ****************************************************************** 
C    *                                                                * 
C    * PERFORM Q-SPACE INTEGRATION FOR COMPUTING THE EELS SPECTRUM OF * 
C    * A ISOTROPIC TARGET USING POLAR COORDINATES.                    * 
C    *                                                                * 
C    * X IS THE DIMENSIONLESS ENERGY LOSS HBAR*OMEGA/(2*E0*PHIA)      * 
C    * AERR AND RERR ARE THE DESIRED ABSOLUTE AND RELATIVE ACCURACIES * 
C    * FACRU*X IS THE UNITS OF WAVEVECTORS OMEGA/V_PERPENDICULAR      * 
C    * F IS THE Q-INTEGRAL MULTIPLIED BY (2/PI)**2                    * 
C    *                                                                * 
C    ****************************************************************** 
      LOGICAL RATION,USER                                               
      DOUBLE PRECISION ACOEF,AERR,BCOEF,CCOEF,COSPSI,DLIMF,ELLEPS,      
     ,ERROR,F,FACRU,FINT1,FINT2,FINT3,FLAG,RERR,RU,SINPSI,U1,U2,UM,UT,  
     ,TANPSI,WN,X,Y                                                     
      DIMENSION ERROR(3),FLAG(3)                                        
      COMMON/PARAM/ACOEF,BCOEF,CCOEF,ELLEPS,COSPSI,SINPSI,TANPSI,       
     ,             RU,UM,DLIMF,WN,USER,RATION                           
      EXTERNAL FINT1,FINT2,FINT3                                        
      F = 0.0D0                                                         
      IF(X.LE.0.0D0) RETURN                                             
      RU = FACRU*X                                                      
      CCOEF = COSPSI**2/X                                               
      UT = CCOEF-BCOEF                                                  
      U1 = ABS(UT)                                                      
      U2 = CCOEF+BCOEF                                                  
      IF(UT.GT.0.0D0) THEN                                              
         CALL QUANC8(FINT1,0.0D0,U1,AERR,RERR,Y,ERROR(1),NOFU,FLAG(1))  
         F = Y                                                          
      ELSE                                                              
         FLAG(1) = 0.0D0                                                
      ENDIF                                                             
      IF(U2.GT.U1) THEN                                                 
         CALL QUANC8(FINT2,U1,U2,AERR,RERR,Y,ERROR(2),NOFU,FLAG(2))     
         F = F+Y                                                        
      ELSE                                                              
         FLAG(2) = 0.0D0                                                
      ENDIF                                                             
      IF(ABS(ACOEF).GT.X*(1.0D0-ELLEPS)*BCOEF) THEN                     
         UM = SQRT(CCOEF/X/(1.0D0-ELLEPS)+BCOEF**2/ACOEF)               
         IF(UM.GT.U2) THEN                                              
            CALL QUANC8(FINT3,U2,UM,AERR,RERR,Y,ERROR(3),NOFU,FLAG(3))  
            F = F+Y                                                     
         ENDIF                                                          
         IF(UM.LT.U1) THEN                                              
            CALL QUANC8(FINT3,UM,U1,AERR,RERR,Y,ERROR(3),NOFU,FLAG(3))  
            F = F-Y                                                     
         ENDIF                                                          
      ELSE                                                              
         FLAG(3) = 0.0D0                                                
      ENDIF                                                             
      ISTOP = 0                                                         
      DO 5 IE=1,3                                                       
         IF(FLAG(IE).EQ.0.0D0) GOTO 5                                   
         WRITE(6,100) IE,FLAG(IE),ERROR(IE)                             
         IF(FLAG(IE)-AINT(FLAG(IE)).GT.0.5D-02) ISTOP = ISTOP+1         
    5 CONTINUE                                                          
      IF(ISTOP.GT.0) STOP '*** EXECUTION ABORTED ***'                   
      F = (2.0D0/3.141592653589793238D0)**2*F                           
      RETURN                                                            
  100 FORMAT(' +++ FLAG(',I1,') =',F10.5,',  ERROR =',D11.4,' +++')     
      END                                                               
